import abjad


def test_Parentage__id_string_01():
    """
    Returns component name if it exists. Otherwise Python ID.
    """

    staff = abjad.Staff("c'8 d'8 e'8 f'8")
    parentage = abjad.get.parentage(staff)
    assert parentage._id_string(staff).startswith("Staff-")


def test_Parentage__id_string_02():
    """
    Returns component name if it exists. Otherwise Python ID.
    """

    staff = abjad.Staff("c'8 d'8 e'8 f'8")
    parentage = abjad.get.parentage(staff)
    staff.name = "foo"
    assert parentage._id_string(staff) == "Staff-'foo'"


def test_Parentage_logical_voice_01():
    """
    An anonymous staff and its contained unvoiced leaves share the same
    signature.
    """

    staff = abjad.Staff("c'8 d'8 e'8 f'8")

    containment = abjad.get.parentage(staff).logical_voice()
    for component in abjad.iterate.components(staff):
        assert abjad.get.parentage(component).logical_voice() == containment


def test_Parentage_logical_voice_02():
    """
    A named staff and its contained unvoiced leaves share the same signature.
    """

    staff = abjad.Staff("c'8 d'8 e'8 f'8")
    staff.name = "foo"

    containment = abjad.get.parentage(staff).logical_voice()
    for component in abjad.iterate.components(staff):
        assert abjad.get.parentage(component).logical_voice() == containment


def test_Parentage_logical_voice_03():
    """
    Leaves inside equally named sequential voices inside a staff share the
    same signature.
    """

    staff = abjad.Staff(
        [abjad.Voice("c'8 d'8 e'8 f'8"), abjad.Voice("c'8 d'8 e'8 f'8")]
    )
    staff[0].name = "foo"
    staff[1].name = "foo"

    containment = abjad.get.parentage(staff[0][0]).logical_voice()
    for leaf in abjad.iterate.leaves(staff):
        assert abjad.get.parentage(leaf).logical_voice() == containment


def test_Parentage_logical_voice_04():
    """
    Returns logical voice giving the root and first voice, staff and score in
    the parentage of component.
    """

    voice = abjad.Voice(
        r"""
        c'8
        d'8
        <<
            \new Voice {
                e'8
                f'8
            }
            \new Voice {
                g'8
                a'8
            }
        >>
        b'8
        c''8
        """
    )

    abjad.override(voice).NoteHead.color = "#red"

    assert abjad.lilypond(voice) == abjad.string.normalize(
        r"""
        \new Voice
        \with
        {
            \override NoteHead.color = #red
        }
        {
            c'8
            d'8
            <<
                \new Voice
                {
                    e'8
                    f'8
                }
                \new Voice
                {
                    g'8
                    a'8
                }
            >>
            b'8
            c''8
        }
        """
    )

    signatures = [
        abjad.get.parentage(leaf).logical_voice()
        for leaf in abjad.iterate.leaves(voice)
    ]

    assert signatures[0] == signatures[1]
    assert signatures[0] != signatures[2]
    assert signatures[0] != signatures[4]
    assert signatures[0] == signatures[6]

    assert signatures[2] == signatures[3]
    assert signatures[2] != signatures[4]


def test_Parentage_logical_voice_05():
    """
    Returns logical voice giving the root and first voice, staff and score in
    parentage of component.
    """

    voice = abjad.Voice(
        r"""
        c'8
        d'8
        <<
            \context Voice = "foo" {
                e'8
                f'8
            }
            \new Voice {
                g'8
                a'8
            }
        >>
        b'8
        c''8
        """
    )

    abjad.override(voice).NoteHead.color = "#red"
    voice.name = "foo"

    assert abjad.lilypond(voice) == abjad.string.normalize(
        r"""
        \context Voice = "foo"
        \with
        {
            \override NoteHead.color = #red
        }
        {
            c'8
            d'8
            <<
                \context Voice = "foo"
                {
                    e'8
                    f'8
                }
                \new Voice
                {
                    g'8
                    a'8
                }
            >>
            b'8
            c''8
        }
        """
    )

    signatures = [
        abjad.get.parentage(leaf).logical_voice()
        for leaf in abjad.iterate.leaves(voice)
    ]

    signatures[0] == signatures[1]
    signatures[0] == signatures[2]
    signatures[0] != signatures[4]
    signatures[0] == signatures[6]

    signatures[2] == signatures[0]
    signatures[2] == signatures[3]
    signatures[2] == signatures[4]
    signatures[2] == signatures[6]

    signatures[4] != signatures[0]
    signatures[4] != signatures[2]
    signatures[4] == signatures[5]
    signatures[4] == signatures[6]


def test_Parentage_logical_voice_06():
    """
    Returns logical voice giving the root and first voice, staff and score in
    parentage of component.
    """

    container = abjad.Container(
        [abjad.Staff([abjad.Voice("c'8 d'8")]), abjad.Staff([abjad.Voice("e'8 f'8")])]
    )
    container[0].name = "staff1"
    container[1].name = "staff2"
    container[0][0].name = "voicefoo"
    container[1][0].name = "voicefoo"
    leaves = abjad.select.leaves(container)
    abjad.beam(leaves[:2])
    abjad.beam(leaves[2:])

    assert abjad.lilypond(container) == abjad.string.normalize(
        r"""
        {
            \context Staff = "staff1"
            {
                \context Voice = "voicefoo"
                {
                    c'8
                    [
                    d'8
                    ]
                }
            }
            \context Staff = "staff2"
            {
                \context Voice = "voicefoo"
                {
                    e'8
                    [
                    f'8
                    ]
                }
            }
        }
        """
    )

    signatures = [abjad.get.parentage(leaf).logical_voice() for leaf in leaves]

    signatures[0] == signatures[1]
    signatures[0] != signatures[2]

    signatures[2] != signatures[2]
    signatures[2] == signatures[3]


def test_Parentage_logical_voice_07():
    """
    Returns logical voice giving the root and first voice, staff and score in
    parentage of component.
    """

    container = abjad.Container(
        r"""
        c'8
        <<
            \context Voice = "alto" {
                d'8
            }
            \context Voice = "soprano" {
                e'8
            }
        >>
        {
            \context Voice = "alto" {
                f'8
            }
            \context Voice = "soprano" {
                g'8
            }
        }
        a'8
        """
    )

    abjad.override(container[1][1]).NoteHead.color = "#red"
    abjad.override(container[2][1]).NoteHead.color = "#red"

    assert abjad.lilypond(container) == abjad.string.normalize(
        r"""
        {
            c'8
            <<
                \context Voice = "alto"
                {
                    d'8
                }
                \context Voice = "soprano"
                \with
                {
                    \override NoteHead.color = #red
                }
                {
                    e'8
                }
            >>
            {
                \context Voice = "alto"
                {
                    f'8
                }
                \context Voice = "soprano"
                \with
                {
                    \override NoteHead.color = #red
                }
                {
                    g'8
                }
            }
            a'8
        }
        """
    )

    signatures = [
        abjad.get.parentage(leaf).logical_voice()
        for leaf in abjad.iterate.leaves(container)
    ]

    signatures[0] != signatures[1]
    signatures[0] != signatures[2]
    signatures[0] != signatures[3]
    signatures[0] != signatures[4]
    signatures[0] == signatures[5]

    signatures[1] != signatures[0]
    signatures[1] != signatures[2]
    signatures[1] == signatures[3]
    signatures[1] != signatures[4]
    signatures[1] != signatures[5]

    signatures[2] != signatures[0]
    signatures[2] != signatures[1]
    signatures[2] != signatures[3]
    signatures[2] == signatures[4]
    signatures[2] != signatures[5]


def test_Parentage_logical_voice_08():
    """
    Unicorporated leaves carry equivalent containment signatures.
    """

    note_1 = abjad.Note(0, (1, 8))
    note_2 = abjad.Note(0, (1, 8))

    signature_1 = abjad.get.parentage(note_1).logical_voice()
    signature_2 = abjad.get.parentage(note_2).logical_voice()
    assert signature_1 == signature_2


def test_Parentage_logical_voice_09():
    """
    Notes appear in the same logical voice.
    """

    staff_1 = abjad.Staff([abjad.Voice([abjad.Note(0, (1, 8))])])
    staff_1.name = "staff"
    staff_1[0].name = "voice"

    staff_2 = abjad.Staff([abjad.Voice([abjad.Note(0, (1, 8))])])
    staff_2.name = "staff"
    staff_2[0].name = "voice"

    staff_1_leaf_signature = abjad.get.parentage(staff_1[0][0]).logical_voice()
    staff_2_leaf_signature = abjad.get.parentage(staff_2[0][0]).logical_voice()
    assert staff_1_leaf_signature == staff_2_leaf_signature


def test_Parentage_logical_voice_10():
    """
    Measure and leaves must carry same logical voice signature.
    """

    staff = abjad.Staff(
        r"""
        {
            \time 2/8
            c'8
            d'8
        }
        e'8
        f'8
        """
    )
    abjad.Score([staff], name="Score")

    assert abjad.lilypond(staff) == abjad.string.normalize(
        r"""
        \new Staff
        {
            {
                \time 2/8
                c'8
                d'8
            }
            e'8
            f'8
        }
        """
    )

    assert (
        abjad.get.parentage(staff[0]).logical_voice()
        == abjad.get.parentage(staff[-1]).logical_voice()
    )
    assert (
        abjad.get.parentage(staff[0]).logical_voice()
        == abjad.get.parentage(staff[0][0]).logical_voice()
    )
    assert (
        abjad.get.parentage(staff[0][0]).logical_voice()
        == abjad.get.parentage(staff[-1]).logical_voice()
    )


def test_Parentage_logical_voice_11():
    """
    Leaves inside different staves have different logical voice signatures,
    even when the staves have the same name.
    """

    container = abjad.Container([abjad.Staff("c'8 c'8"), abjad.Staff("c'8 c'8")])
    container[0].name = container[1].name = "staff"

    assert abjad.lilypond(container) == abjad.string.normalize(
        r"""
        {
            \context Staff = "staff"
            {
                c'8
                c'8
            }
            \context Staff = "staff"
            {
                c'8
                c'8
            }
        }
        """
    )

    leaves = abjad.select.leaves(container)
    assert (
        abjad.get.parentage(leaves[0]).logical_voice()
        == abjad.get.parentage(leaves[1]).logical_voice()
    )
    assert (
        abjad.get.parentage(leaves[0]).logical_voice()
        != abjad.get.parentage(leaves[2]).logical_voice()
    )
    assert (
        abjad.get.parentage(leaves[2]).logical_voice()
        == abjad.get.parentage(leaves[3]).logical_voice()
    )
    assert (
        abjad.get.parentage(leaves[2]).logical_voice()
        != abjad.get.parentage(leaves[0]).logical_voice()
    )


def test_Parentage_orphan_01():
    staff = abjad.Staff("c'8 d'8 e'8 f'8")

    assert abjad.get.parentage(staff).orphan
    for note in staff:
        assert not abjad.get.parentage(note).orphan


def test_Parentage_root_01():
    tuplet = abjad.Tuplet((2, 3), "c'8 d'8 e'8")
    staff = abjad.Staff([tuplet])
    leaves = abjad.select.leaves(staff)

    assert abjad.get.parentage(staff).root is staff
    assert abjad.get.parentage(tuplet).root is staff
    assert abjad.get.parentage(leaves[0]).root is staff
    assert abjad.get.parentage(leaves[1]).root is staff
    assert abjad.get.parentage(leaves[2]).root is staff



import pytest

import abjad

values = [
    (-1, -1),
    (-2, -2),
    (-3, -3),
    (-4, -4),
    (0, 0),
    (1, 1),
    (2, 2),
    (3, 3),
    (4, 4),
    (abjad.Octave(), 4),
    (abjad.Octave(3), 3),
    (abjad.Octave(5), 5),
]


@pytest.mark.parametrize("input_, expected_number", values)
def test_init(input_, expected_number):
    if isinstance(expected_number, type) and issubclass(expected_number, Exception):
        with pytest.raises(expected_number):
            abjad.Octave(input_)
        return
    octave = abjad.Octave(input_)
    assert octave.number == expected_number


values = [
    ("'", 4),
    ("''", 5),
    ("'''", 6),
    ("''''", 7),
    ("", 3),
    (",", 2),
    (",,", 1),
    (",,,", 0),
]


@pytest.mark.parametrize("input_, expected_number", values)
def test_from_ticks(input_, expected_number):
    if isinstance(expected_number, type) and issubclass(expected_number, Exception):
        with pytest.raises(expected_number):
            abjad.Octave.from_ticks(input_)
        return
    octave = abjad.Octave.from_ticks(input_)
    assert octave.number == expected_number


values = [
    ("bf,", 2),
    ("c'", 4),
    ("cs'", 4),
    ("gff''", 5),
    ("dss,,", 1),
    (("bf", 2), 2),
    (("c", 4), 4),
    (("cs", 4), 4),
    (("dss", 1), 1),
    (("gff", 5), 5),
    (abjad.NamedPitch("bs'"), 4),
    (abjad.NamedPitch("c"), 3),
    (abjad.NamedPitch("cf,"), 2),
    (abjad.NamedPitch(), 4),
    (abjad.NamedPitchClass("cs'"), 4),
    (abjad.NamedPitchClass("c"), 4),
    (abjad.NamedPitchClass("cf,"), 4),
    (abjad.NumberedPitch("bs'"), 5),
    (abjad.NumberedPitch("c"), 3),
    (abjad.NumberedPitch("cf,"), 1),
    (abjad.NumberedPitch(), 4),
    (abjad.NumberedPitchClass("bs'"), 4),
    (abjad.NumberedPitchClass("c"), 4),
    (abjad.NumberedPitchClass("cf,"), 4),
]


@pytest.mark.parametrize("input_, expected_number", values)
def test_init_from_pitch(input_, expected_number):
    octave = abjad.NamedPitch(input_).octave
    assert octave.number == expected_number



import copy

import abjad


def test_Tuplet___copy___01():
    tuplet_1 = abjad.Tuplet((2, 3), "c'8 d'8 e'8")
    abjad.override(tuplet_1).NoteHead.color = "#red"

    assert abjad.lilypond(tuplet_1) == abjad.string.normalize(
        r"""
        \override NoteHead.color = #red
        \tuplet 3/2
        {
            c'8
            d'8
            e'8
        }
        \revert NoteHead.color
        """
    )

    tuplet_2 = copy.copy(tuplet_1)

    assert abjad.lilypond(tuplet_2) == abjad.string.normalize(
        r"""
        \override NoteHead.color = #red
        \tuplet 3/2
        {
        }
        \revert NoteHead.color
        """
    )

    assert not len(tuplet_2)


def test_Tuplet___init___01():
    """
    Initializes tuplet from empty input.
    """

    tuplet = abjad.Tuplet()

    assert abjad.lilypond(tuplet) == "\\tuplet 3/2\n{\n}"
    assert tuplet.multiplier == (2, 3)
    assert not len(tuplet)


# TODO: move to test_get_timespan.py
def test_Tuplet_get_timespan_01():
    staff = abjad.Staff(r"c'4 d'4 \tuplet 3/2 { e'4 f'4 g'4 }")
    leaves = abjad.select.leaves(staff)
    score = abjad.Score([staff])
    mark = abjad.MetronomeMark(abjad.Duration(1, 4), 60)
    abjad.attach(mark, leaves[0])

    assert abjad.lilypond(score) == abjad.string.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \tempo 4=60
                c'4
                d'4
                \tuplet 3/2
                {
                    e'4
                    f'4
                    g'4
                }
            }
        >>
        """
    )

    assert abjad.get.timespan(staff, in_seconds=True) == abjad.Timespan(0, 4)
    assert abjad.get.timespan(staff[0], in_seconds=True) == abjad.Timespan(0, 1)
    assert abjad.get.timespan(staff[1], in_seconds=True) == abjad.Timespan(1, 2)
    assert abjad.get.timespan(staff[-1], in_seconds=True) == abjad.Timespan(2, 4)


def test_Tuplet_set_minimum_denominator_01():
    tuplet = abjad.Tuplet((3, 5), "c'4 d'8 e'8 f'4 g'2")
    tuplet.set_minimum_denominator(8)

    assert abjad.lilypond(tuplet) == abjad.string.normalize(
        r"""
        \tweak text #tuplet-number::calc-fraction-text
        \tuplet 10/6
        {
            c'4
            d'8
            e'8
            f'4
            g'2
        }
        """
    )

    assert abjad.wf.wellformed(tuplet)


def test_Tuplet_set_minimum_denominator_02():
    tuplet = abjad.Tuplet((3, 5), "c'4 d'8 e'8 f'4 g'2")
    tuplet.set_minimum_denominator(16)

    assert abjad.lilypond(tuplet) == abjad.string.normalize(
        r"""
        \tweak text #tuplet-number::calc-fraction-text
        \tuplet 20/12
        {
            c'4
            d'8
            e'8
            f'4
            g'2
        }
        """
    )

    assert abjad.wf.wellformed(tuplet)


def test_Tuplet_set_minimum_denominator_03():
    tuplet = abjad.Tuplet((3, 5), "c'4 d'8 e'8 f'4 g'2")
    tuplet.set_minimum_denominator(2)

    assert abjad.lilypond(tuplet) == abjad.string.normalize(
        r"""
        \tweak text #tuplet-number::calc-fraction-text
        \tuplet 5/3
        {
            c'4
            d'8
            e'8
            f'4
            g'2
        }
        """
    )

    assert abjad.wf.wellformed(tuplet)



import pytest

import abjad


def test_Mutation_swap_01():
    """
    Moves parentage, children from multiple containers to empty tuplet.
    """

    voice = abjad.Voice("{ c'8 d'8 } { e'8 f'8 } { g'8 a'8 }")
    leaves = abjad.select.leaves(voice)
    abjad.beam(leaves)

    assert abjad.lilypond(voice) == abjad.string.normalize(
        r"""
        \new Voice
        {
            {
                c'8
                [
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
                ]
            }
        }
        """
    )

    tuplet = abjad.Tuplet((3, 4), [])
    abjad.mutate.swap(voice[:2], tuplet)

    assert abjad.lilypond(voice) == abjad.string.normalize(
        r"""
        \new Voice
        {
            \tweak text #tuplet-number::calc-fraction-text
            \tuplet 4/3
            {
                c'8
                [
                d'8
                e'8
                f'8
            }
            {
                g'8
                a'8
                ]
            }
        }
        """
    )

    assert abjad.wf.wellformed(voice, check_overlapping_beams=False)


def test_Mutation_swap_02():
    """
    Moves parentage, children from container to empty voice.
    """

    voice = abjad.Voice("{ c'8 d'8 } { e'8 f'8 } { g'8 a'8 }")
    leaves = abjad.select.leaves(voice)
    voice.name = "foo"
    abjad.beam(leaves)

    assert abjad.lilypond(voice) == abjad.string.normalize(
        r"""
        \context Voice = "foo"
        {
            {
                c'8
                [
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
                ]
            }
        }
        """
    )

    new_voice = abjad.Voice()
    new_voice.name = "foo"
    abjad.mutate.swap(voice[1:2], new_voice)

    assert abjad.lilypond(voice) == abjad.string.normalize(
        r"""
        \context Voice = "foo"
        {
            {
                c'8
                [
                d'8
            }
            \context Voice = "foo"
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
                ]
            }
        }
        """
    )

    assert abjad.wf.wellformed(voice)


def test_Mutation_swap_03():
    """
    Moves parentage, children from container to empty tuplet.
    """

    voice = abjad.Voice("{ c'8 d'8 } { e'8 f'8 } { g'8 a'8 }")
    leaves = abjad.select.leaves(voice)
    abjad.beam(leaves)

    assert abjad.lilypond(voice) == abjad.string.normalize(
        r"""
        \new Voice
        {
            {
                c'8
                [
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
                ]
            }
        }
        """
    )

    tuplet = abjad.Tuplet((3, 4), [])
    abjad.mutate.swap(voice[1:2], tuplet)

    assert abjad.lilypond(voice) == abjad.string.normalize(
        r"""
        \new Voice
        {
            {
                c'8
                [
                d'8
            }
            \tweak text #tuplet-number::calc-fraction-text
            \tuplet 4/3
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
                ]
            }
        }
        """
    )

    assert abjad.wf.wellformed(voice)


def test_Mutation_swap_04():
    """
    Trying to move parentage, children to noncontainer raises exception.
    """

    voice = abjad.Voice("{ c'8 d'8 } { e'8 f'8 }")
    leaves = abjad.select.leaves(voice)
    abjad.beam(leaves)

    note = abjad.Note("c'4")
    with pytest.raises(Exception):
        abjad.mutate.swap(voice[1:2], note)


def test_Mutation_swap_05():
    """
    Trying to move parentage, children from nonempty container to nonempty
    container raises exception.
    """

    voice = abjad.Voice("{ c'8 d'8 } { e'8 f'8 }")
    leaves = abjad.select.leaves(voice)
    abjad.beam(leaves)

    tuplet = abjad.Tuplet((2, 3), "c'8 d'8 e'8")
    with pytest.raises(Exception):
        abjad.mutate.swap(voice[1:2], tuplet)


def test_Mutation_swap_06():
    """
    Trying to move parentage, children from components that are not
    parent-contiguous raises exception.
    """

    voice = abjad.Voice("{ c'8 d'8 } { e'8 f'8 } { g'8 a'8 }")
    leaves = abjad.select.leaves(voice)
    abjad.beam(leaves)

    assert abjad.lilypond(voice) == abjad.string.normalize(
        r"""
        \new Voice
        {
            {
                c'8
                [
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
                ]
            }
        }
        """
    )

    tuplet = abjad.Tuplet((2, 3), [])
    with pytest.raises(Exception):
        abjad.mutate.swap([voice[0], voice[2]], tuplet)


def test_Mutation_swap_07():
    """
    Moves parentage, children from one container to another.
    """

    measure = abjad.Container("c'8 d'8 e'8 f'8")

    assert abjad.lilypond(measure) == abjad.string.normalize(
        r"""
        {
            c'8
            d'8
            e'8
            f'8
        }
        """
    )

    new_measure = abjad.Container()
    abjad.mutate.swap(measure, new_measure)

    assert abjad.lilypond(new_measure) == abjad.string.normalize(
        r"""
        {
            c'8
            d'8
            e'8
            f'8
        }
        """
    )

    assert abjad.wf.wellformed(new_measure)



import pathlib
import re
import shutil

import pytest
from uqbar.strings import normalize


@pytest.fixture
def rm_dirs(app):
    path = pathlib.Path(app.doctreedir)
    shutil.rmtree(path)
    yield


@pytest.mark.sphinx("html", testroot="ext-sphinx")
def test_ext_sphinx_01(app, status, warning, rm_dirs):
    app.build()
    assert not warning.getvalue().strip()
    images_path = pathlib.Path(app.outdir) / "_images"
    assert sorted(path.name for path in images_path.iterdir()) == [
        "lilypond-2065254c213caa1da7f276b113c411fe735780505cfebb626eb6af1832248d37.cropped.svg",
        "lilypond-2065254c213caa1da7f276b113c411fe735780505cfebb626eb6af1832248d37.ly",
        "lilypond-2065254c213caa1da7f276b113c411fe735780505cfebb626eb6af1832248d37.svg",
        "lilypond-2e1d3707c4c266f8abec146c8af80dd191b7ab568a99eb82417c2bed8a8872ef-1.svg",
        "lilypond-2e1d3707c4c266f8abec146c8af80dd191b7ab568a99eb82417c2bed8a8872ef-2.svg",
        "lilypond-2e1d3707c4c266f8abec146c8af80dd191b7ab568a99eb82417c2bed8a8872ef-3.svg",
        "lilypond-2e1d3707c4c266f8abec146c8af80dd191b7ab568a99eb82417c2bed8a8872ef-4.svg",
        "lilypond-2e1d3707c4c266f8abec146c8af80dd191b7ab568a99eb82417c2bed8a8872ef.cropped.svg",
        "lilypond-2e1d3707c4c266f8abec146c8af80dd191b7ab568a99eb82417c2bed8a8872ef.ly",
    ]

    index_path = pathlib.Path(app.srcdir) / "_build" / "html" / "index.html"
    index_source = index_path.read_text()
    assert re.findall(r"lilypond-\w{64}(?:-\d+|.cropped)?\.svg", index_source) == [
        "lilypond-2065254c213caa1da7f276b113c411fe735780505cfebb626eb6af1832248d37.cropped.svg",
        "lilypond-2065254c213caa1da7f276b113c411fe735780505cfebb626eb6af1832248d37.svg",
        "lilypond-2e1d3707c4c266f8abec146c8af80dd191b7ab568a99eb82417c2bed8a8872ef.cropped.svg",
        "lilypond-2e1d3707c4c266f8abec146c8af80dd191b7ab568a99eb82417c2bed8a8872ef-1.svg",
        "lilypond-2e1d3707c4c266f8abec146c8af80dd191b7ab568a99eb82417c2bed8a8872ef-2.svg",
        "lilypond-2e1d3707c4c266f8abec146c8af80dd191b7ab568a99eb82417c2bed8a8872ef-3.svg",
        "lilypond-2e1d3707c4c266f8abec146c8af80dd191b7ab568a99eb82417c2bed8a8872ef-4.svg",
        "lilypond-2e1d3707c4c266f8abec146c8af80dd191b7ab568a99eb82417c2bed8a8872ef-2.svg",
        "lilypond-2e1d3707c4c266f8abec146c8af80dd191b7ab568a99eb82417c2bed8a8872ef-3.svg",
        "lilypond-2e1d3707c4c266f8abec146c8af80dd191b7ab568a99eb82417c2bed8a8872ef-1.svg",
        "lilypond-2e1d3707c4c266f8abec146c8af80dd191b7ab568a99eb82417c2bed8a8872ef-1.svg",
        "lilypond-2e1d3707c4c266f8abec146c8af80dd191b7ab568a99eb82417c2bed8a8872ef-1.svg",
        "lilypond-2e1d3707c4c266f8abec146c8af80dd191b7ab568a99eb82417c2bed8a8872ef-2.svg",
        "lilypond-2e1d3707c4c266f8abec146c8af80dd191b7ab568a99eb82417c2bed8a8872ef-2.svg",
        "lilypond-2e1d3707c4c266f8abec146c8af80dd191b7ab568a99eb82417c2bed8a8872ef-3.svg",
        "lilypond-2e1d3707c4c266f8abec146c8af80dd191b7ab568a99eb82417c2bed8a8872ef-3.svg",
        "lilypond-2e1d3707c4c266f8abec146c8af80dd191b7ab568a99eb82417c2bed8a8872ef-4.svg",
        "lilypond-2e1d3707c4c266f8abec146c8af80dd191b7ab568a99eb82417c2bed8a8872ef-4.svg",
    ]


@pytest.mark.sphinx("text", testroot="ext-sphinx")
def test_ext_sphinx_02(app, status, warning, rm_dirs):
    app.build()
    assert not warning.getvalue().strip()
    index_path = pathlib.Path(app.srcdir) / "_build" / "text" / "index.txt"
    assert normalize(index_path.read_text()) == normalize(
        r"""
        Fake Docs
        *********

        This will show the cropped version of the staff:

           >>> staff = abjad.Staff("c' d' e' f'")
           >>> abjad.show(staff)

           \version "2.19.83"
           \language "english"
           \score
           {
               \new Staff
               {
                   c'4
                   d'4
                   e'4
                   f'4
               }
           }

        This will show the uncropped version of the staff:

           >>> abjad.show(staff)

           \version "2.19.83"
           \language "english"
           \score
           {
               \new Staff
               {
                   c'4
                   d'4
                   e'4
                   f'4
               }
           }

        This will show the single-SVG cropped version of the 4-page staff:

           >>> multipage_staff = abjad.Staff("c'1 d'1 e'1 f'1")
           >>> for note in multipage_staff:
           ...     page_break = abjad.LilyPondLiteral(r"\pageBreak", site="after")
           ...     abjad.attach(page_break, note)
           ...
           >>> abjad.show(multipage_staff)

           \version "2.19.83"
           \language "english"
           \score
           {
               \new Staff
               {
                   c'1
                   \pageBreak
                   d'1
                   \pageBreak
                   e'1
                   \pageBreak
                   f'1
                   \pageBreak
               }
           }

        This will show four individual pages, because there is no single SVG
        for uncropped output for the 4-page staff:

           >>> abjad.show(multipage_staff)

           \version "2.19.83"
           \language "english"
           \score
           {
               \new Staff
               {
                   c'1
                   \pageBreak
                   d'1
                   \pageBreak
                   e'1
                   \pageBreak
                   f'1
                   \pageBreak
               }
           }

        This will show pages 2, 3 and 1 of the 4-page staff:

           >>> abjad.show(multipage_staff)

           \version "2.19.83"
           \language "english"
           \score
           {
               \new Staff
               {
                   c'1
                   \pageBreak
                   d'1
                   \pageBreak
                   e'1
                   \pageBreak
                   f'1
                   \pageBreak
               }
           }

        This will show all four pages, in a 2x2 grid:

           >>> abjad.show(multipage_staff)

           \version "2.19.83"
           \language "english"
           \score
           {
               \new Staff
               {
                   c'1
                   \pageBreak
                   d'1
                   \pageBreak
                   e'1
                   \pageBreak
                   f'1
                   \pageBreak
               }
           }
        """
    )



from .. import _iterlib
from .. import bind as _bind
from .. import duration as _duration
from .. import enums as _enums
from .. import exceptions as _exceptions
from .. import indicators as _indicators
from .. import pitch as _pitch
from .. import score as _score
from .. import select as _select
from .base import Parser


class ReducedLyParser(Parser):
    r"""
    Parses the "reduced-ly" syntax, a modified subset of LilyPond syntax.

    ..  container:: example

        >>> from abjad.parsers.reduced import ReducedLyParser
        >>> parser = ReducedLyParser()

        Understands LilyPond-like representation of notes, chords and rests:

        >>> string = "c'4 r8. <b d' fs'>16"
        >>> container = parser(string)
        >>> abjad.show(container) # doctest: +SKIP

        ..  docs::

            >>> string = abjad.lilypond(container)
            >>> print(string)
            {
                c'4
                r8.
                <b d' fs'>16
            }

        Also parses bare duration as notes on middle-C, and negative bare
        durations as rests:

        >>> string = '4 -8 16. -32'
        >>> container = parser(string)
        >>> abjad.show(container) # doctest: +SKIP

        ..  docs::

            >>> string = abjad.lilypond(container)
            >>> print(string)
            {
                c'4
                r8
                c'16.
                r32
            }

    ..  container:: example

        Note that the leaf syntax is greedy, and therefore duration specifiers
        following pitch specifiers will be treated as part of the same
        expression. The following produces 2 leaves, rather than 3:

        >>> string = "4 d' 4"
        >>> container = parser(string)
        >>> abjad.show(container) # doctest: +SKIP

        ..  docs::

            >>> string = abjad.lilypond(container)
            >>> print(string)
            {
                c'4
                d'4
            }

        Understands LilyPond-like default durations:

        >>> string = "c'4 d' e' f'"
        >>> container = parser(string)
        >>> abjad.show(container) # doctest: +SKIP

        ..  docs::

            >>> string = abjad.lilypond(container)
            >>> print(string)
            {
                c'4
                d'4
                e'4
                f'4
            }

    Also understands various types of container specifications.

    ..  container:: example

        Can create arbitrarily nested tuplets:

        >>> string = "2/3 { 4 4 3/5 { 8 8 8 } }"
        >>> tuplet = parser(string)
        >>> abjad.show(tuplet) # doctest: +SKIP

        ..  docs::

            >>> string = abjad.lilypond(tuplet)
            >>> print(string)
            \tweak edge-height #'(0.7 . 0)
            \tuplet 3/2
            {
                c'4
                c'4
                \tweak text #tuplet-number::calc-fraction-text
                \tweak edge-height #'(0.7 . 0)
                \tuplet 5/3
                {
                    c'8
                    c'8
                    c'8
                }
            }

    ..  container:: example

        Can create measures too:

        >>> string = '| 4/4 4 4 4 4 || 3/8 8 8 8 |'
        >>> container = parser(string)
        >>> staff = abjad.Staff([container])
        >>> score = abjad.Score([staff], name="Score")
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> string = abjad.lilypond(staff)
            >>> print(string)
            \new Staff
            {
                {
                    {
                        \time 4/4
                        c'4
                        c'4
                        c'4
                        c'4
                    }
                    {
                        \time 3/8
                        c'8
                        c'8
                        c'8
                    }
                }
            }

    ..  container:: example

        Finally, understands ties, slurs and beams:

        >>> string = 'c16 [ ( d ~ d ) f ]'
        >>> container = parser(string)
        >>> abjad.show(container) # doctest: +SKIP

        ..  docs::

            >>> string = abjad.lilypond(container)
            >>> print(string)
            {
                c16
                [
                (
                d16
                ~
                d16
                )
                f16
                ]
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = ("_default_duration", "_toplevel_component_count")

    ### INITIALIZER ###

    def __init__(self, debug=False):
        self._default_duration = _duration.Duration((1, 4))
        self._toplevel_component_count = None
        Parser.__init__(self, debug=debug)

    ### LEX SETUP ###

    tokens = (
        "APOSTROPHE",
        "BRACE_L",
        "BRACE_R",
        "BRACKET_L",
        "BRACKET_R",
        "CARAT_L",
        "CARAT_R",
        "COMMA",
        "DOT",
        "FRACTION",
        "INTEGER_N",
        "INTEGER_P",
        "PAREN_L",
        "PAREN_R",
        "PIPE",
        "PITCHNAME",
        "RESTNAME",
        "TILDE",
    )

    t_APOSTROPHE = "'"
    t_BRACE_L = "{"
    t_BRACE_R = "}"
    t_BRACKET_L = r"\["
    t_BRACKET_R = r"\]"
    t_CARAT_L = r"\<"
    t_CARAT_R = r"\>"
    t_COMMA = ","
    t_DOT = r"\."
    t_PAREN_L = r"\("
    t_PAREN_R = r"\)"
    t_PIPE = r"\|"
    t_RESTNAME = "r"
    t_TILDE = "~"

    t_ignore = " \t\r"

    ### YACC SETUP ###

    start = "start"

    ### LEX METHODS ###

    def t_FRACTION(self, t):
        r"([1-9]\d*/[1-9]\d*)"
        parts = t.value.split("/")
        t.value = int(parts[0]), int(parts[1])
        return t

    def t_INTEGER_N(self, t):
        r"(-[1-9]\d*)"
        t.value = int(t.value)
        return t

    def t_INTEGER_P(self, t):
        r"([1-9]\d*)"
        t.value = int(t.value)
        return t

    def t_PITCHNAME(self, t):
        r"[a-g](ff|ss|f|s|tqf|tqs|qs|qf)?"
        t.value = _pitch.NamedPitchClass(t.value)
        return t

    def t_error(self, t):
        print(("Illegal character '%s'" % t.value[0]))
        t.lexer.skip(1)

    def t_newline(self, t):
        r"\n+"
        t.lexer.lineno += t.value.count("\n")

    ### YACC METHODS ###

    def p_apostrophes__APOSTROPHE(self, p):
        """
        apostrophes : APOSTROPHE
        """
        p[0] = 1

    def p_apostrophes__apostrophes__APOSTROPHE(self, p):
        """
        apostrophes : apostrophes APOSTROPHE
        """
        p[0] = p[1] + 1

    def p_beam__BRACKET_L(self, p):
        """
        beam : BRACKET_L
        """
        p[0] = (_indicators.StartBeam, _enums.LEFT)

    def p_beam__BRACKET_R(self, p):
        """
        beam : BRACKET_R
        """
        p[0] = (_indicators.StopBeam, _enums.RIGHT)

    def p_chord_body__chord_pitches(self, p):
        """
        chord_body : chord_pitches
        """
        p[0] = _score.Chord(p[1], self._default_duration)

    def p_chord_body__chord_pitches__positive_leaf_duration(self, p):
        """
        chord_body : chord_pitches positive_leaf_duration
        """
        p[0] = _score.Chord(p[1], p[2])

    def p_chord_pitches__CARAT_L__pitches__CARAT_R(self, p):
        """
        chord_pitches : CARAT_L pitches CARAT_R
        """
        p[0] = p[2]

    def p_commas__COMMA(self, p):
        """
        commas : COMMA
        """
        p[0] = 1

    def p_commas__commas__commas(self, p):
        """
        commas : commas COMMA
        """
        p[0] = p[1] + 1

    def p_component__container(self, p):
        """
        component : container
        """
        p[0] = p[1]

    def p_component__fixed_duration_container(self, p):
        """
        component : fixed_duration_container
        """
        p[0] = p[1]

    def p_component__leaf(self, p):
        """
        component : leaf
        """
        p[0] = p[1]

    def p_component__tuplet(self, p):
        """
        component : tuplet
        """
        p[0] = p[1]

    def p_component_list__EMPTY(self, p):
        """
        component_list :
        """
        p[0] = []

    def p_component_list__component_list__component(self, p):
        """
        component_list : component_list component
        """
        p[0] = p[1] + [p[2]]

    def p_container__BRACE_L__component_list__BRACE_R(self, p):
        r"""container : BRACE_L component_list BRACE_R"""
        p[0] = _score.Container()
        for component in p[2]:
            p[0].append(component)

    def p_dots__EMPTY(self, p):
        """
        dots :
        """
        p[0] = 0

    def p_dots__dots__DOT(self, p):
        """
        dots : dots DOT
        """
        p[0] = p[1] + 1

    def p_error(self, p):
        if p:
            print(("Syntax error at '%s'" % p.value))
        else:
            print("Syntax error at EOF")

    def p_fixed_duration_container__BRACE_L__FRACTION__BRACE_R(self, p):
        """
        fixed_duration_container : BRACE_L FRACTION BRACE_R
        """
        raise Exception("fixed-duration containers no longer supported.")

    def p_leaf__leaf_body__post_events(self, p):
        """
        leaf : leaf_body post_events
        """
        p[0] = p[1]
        if p[2]:
            annotation = {"post events": p[2]}
            _bind._unsafe_attach(annotation, p[0])

    def p_leaf_body__chord_body(self, p):
        """
        leaf_body : chord_body
        """
        p[0] = p[1]

    def p_leaf_body__note_body(self, p):
        """
        leaf_body : note_body
        """
        p[0] = p[1]

    def p_leaf_body__rest_body(self, p):
        """
        leaf_body : rest_body
        """
        p[0] = p[1]

    def p_measure__PIPE__FRACTION__component_list__PIPE(self, p):
        """
        measure : PIPE FRACTION component_list PIPE
        """
        measure = _score.Container()
        for x in p[3]:
            measure.append(x)
        leaf = _iterlib._get_leaf(measure, 0)
        time_signature = _indicators.TimeSignature(p[2])
        try:
            _bind._unsafe_attach(time_signature, leaf)
        except _exceptions.MissingContextError:
            score = _score.Score([measure])
            _bind._unsafe_attach(time_signature, leaf)
            score[:] = []
        p[0] = measure

    def p_negative_leaf_duration__INTEGER_N__dots(self, p):
        """
        negative_leaf_duration : INTEGER_N dots
        """
        duration_log = p[1]
        dots = "." * p[2]
        string = f"{abs(duration_log)}{dots}"
        duration = _duration.Duration.from_lilypond_duration_string(string)
        self._default_duration = duration
        p[0] = duration

    def p_note_body__pitch(self, p):
        """
        note_body : pitch
        """
        p[0] = _score.Note(p[1], self._default_duration)

    def p_note_body__pitch__positive_leaf_duration(self, p):
        """
        note_body : pitch positive_leaf_duration
        """
        p[0] = _score.Note(p[1], p[2])

    def p_note_body__positive_leaf_duration(self, p):
        """
        note_body : positive_leaf_duration
        """
        p[0] = _score.Note(0, p[1])

    def p_pitch__PITCHNAME(self, p):
        """
        pitch : PITCHNAME
        """
        p[0] = _pitch.NamedPitch(p[1].name)

    def p_pitch__PITCHNAME__apostrophes(self, p):
        """
        pitch : PITCHNAME apostrophes
        """
        p[0] = _pitch.NamedPitch(p[1].name + "'" * p[2])

    def p_pitch__PITCHNAME__commas(self, p):
        """
        pitch : PITCHNAME commas
        """
        p[0] = _pitch.NamedPitch(p[1].name + "," * p[2])

    def p_pitches__pitch(self, p):
        """
        pitches : pitch
        """
        p[0] = [p[1]]

    def p_pitches__pitches__pitch(self, p):
        """
        pitches : pitches pitch
        """
        p[0] = p[1] + [p[2]]

    def p_positive_leaf_duration__INTEGER_P__dots(self, p):
        """
        positive_leaf_duration : INTEGER_P dots
        """
        duration_log = p[1]
        dots = "." * p[2]
        duration = _duration.Duration.from_lilypond_duration_string(
            f"{abs(duration_log)}{dots}"
        )
        self._default_duration = duration
        p[0] = duration

    def p_post_event__beam(self, p):
        """
        post_event : beam
        """
        p[0] = p[1]

    def p_post_event__slur(self, p):
        """
        post_event : slur
        """
        p[0] = p[1]

    def p_post_event__tie(self, p):
        """
        post_event : tie
        """
        p[0] = p[1]

    def p_post_events__EMPTY(self, p):
        """
        post_events :
        """
        p[0] = {}

    def p_post_events__post_events__post_event(self, p):
        """
        post_events : post_events post_event
        """
        kind, direction = p[2]
        if kind in p[1]:
            p[1][kind].append(direction)
        else:
            p[1][kind] = [direction]
        p[0] = p[1]

    def p_rest_body__RESTNAME(self, p):
        """
        rest_body : RESTNAME
        """
        p[0] = _score.Rest(self._default_duration)

    def p_rest_body__RESTNAME__positive_leaf_duration(self, p):
        """
        rest_body : RESTNAME positive_leaf_duration
        """
        p[0] = _score.Rest(p[2])

    def p_rest_body__negative_leaf_duration(self, p):
        """
        rest_body : negative_leaf_duration
        """
        p[0] = _score.Rest(p[1])

    def p_slur__PAREN_L(self, p):
        """
        slur : PAREN_L
        """
        p[0] = (_indicators.StartSlur, _enums.LEFT)

    def p_slur__PAREN_R(self, p):
        """
        slur : PAREN_R
        """
        p[0] = (_indicators.StopSlur, _enums.RIGHT)

    def p_start__EMPTY(self, p):
        """
        start :
        """
        self._toplevel_component_count = 0
        p[0] = []

    def p_start__start__component(self, p):
        """
        start : start component
        """
        self._toplevel_component_count += 1
        p[0] = p[1] + [p[2]]

    def p_start__start__measure(self, p):
        """
        start : start measure
        """
        self._toplevel_component_count += 1
        p[0] = p[1] + [p[2]]

    def p_tie__TILDE(self, p):
        """
        tie : TILDE
        """
        p[0] = (_indicators.Tie, _enums.LEFT)

    def p_tuplet__FRACTION__container(self, p):
        """
        tuplet : FRACTION container
        """
        assert isinstance(p[2], _score.Container)
        leaves = p[2][:]
        p[2][:] = []
        p[0] = _score.Tuplet(p[1], leaves)

    ### PRIVATE METHODS ###

    def _attach_indicators(self, leaves):
        for leaf in leaves:
            span_events = self._get_span_events(leaf)
            for current_class, directions in span_events.items():
                if current_class in (
                    _indicators.StartSlur,
                    _indicators.StopSlur,
                ):
                    indicator = current_class()
                    _bind._unsafe_attach(indicator, leaf)
                    continue
                if current_class in (
                    _indicators.StartBeam,
                    _indicators.StopBeam,
                ):
                    indicator = current_class()
                    _bind._unsafe_attach(indicator, leaf)
                    continue
                if current_class is _indicators.Tie:
                    indicator = current_class()
                    _bind._unsafe_attach(indicator, leaf)
                    continue

    def _cleanup(self, parsed):
        container = _score.Container()
        for x in parsed:
            container.append(x)
        parsed = container
        leaves = _select.leaves(parsed)
        if leaves:
            self._attach_indicators(leaves)
        for leaf in leaves:
            _bind.detach(dict, leaf)
        if 1 < self._toplevel_component_count:
            return parsed
        return parsed[0]

    def _get_span_events(self, leaf):
        annotations = leaf._get_indicators(dict)
        _bind.detach(dict, leaf)
        annotations = [x for x in annotations if "post events" in x]
        if annotations:
            return annotations[0]["post events"]
        return {}

    def _setup(self):
        self._toplevel_component_count = 0
        self._default_duration = _duration.Duration((1, 4))

    ### PUBLIC PROPERTIES ###

    @property
    def debug(self):
        """
        Gets debug boolean of reduced ly parser.

        Returns true or false.
        """
        return self._debug

    @property
    def lexer_rules_object(self):
        """
        Lexer rules object of reduced ly parser.
        """
        return self

    @property
    def parser_rules_object(self):
        """
        Parser rules object of reduced ly parser.
        """
        return self


### FUNCTIONS ###


def parse_reduced_ly_syntax(string) -> _score.Container:
    """
    Parse the reduced LilyPond rhythmic syntax:

    ..  container:: example

        >>> string = '4 -4. 8.. 5/3 { } 4'
        >>> container = abjad.parsers.reduced.parse_reduced_ly_syntax(string)
        >>> container
        Container("c'4 r4. c'8.. { 5/3 } c'4")

        >>> for component in container:
        ...     component
        ...
        Note("c'4")
        Rest('r4.')
        Note("c'8..")
        Tuplet('3:5', '')
        Note("c'4")

    """
    return ReducedLyParser()(string)



import abjad


def test_Component__sibling_01():
    """
    Returns component when index is in range.
    """

    staff = abjad.Staff("c' d' e' f'")
    assert staff[1]._sibling(1) is staff[2]
    assert staff[1]._sibling(-1) is staff[0]


def test_Component__sibling_02():
    """
    Returns none when component has no parent.
    """

    staff = abjad.Staff("c' d' e' f'")
    assert staff._sibling(1) is None


def test_Component__sibling_03():
    staff = abjad.Staff(r"c'4 \tuplet 3/2 { d'8 e'8 f'8 } g'2")

    assert abjad.lilypond(staff) == abjad.string.normalize(
        r"""
        \new Staff
        {
            c'4
            \tuplet 3/2
            {
                d'8
                e'8
                f'8
            }
            g'2
        }
        """
    )

    leaves = abjad.select.leaves(staff)
    tuplet = staff[1]

    assert leaves[0]._sibling(-1) is None
    assert leaves[0]._sibling(1) is tuplet

    assert tuplet._sibling(-1) is leaves[0]
    assert tuplet._sibling(1) is leaves[4]

    assert leaves[1]._sibling(-1) is leaves[0]
    assert leaves[1]._sibling(1) is leaves[2]



import abjad


def test_mutate_extract_01():
    """
    Extracts note.
    """

    voice = abjad.Voice("c'8 d'8 e'8 f'8")
    abjad.beam(voice[:])
    abjad.glissando(voice[:])

    assert abjad.lilypond(voice) == abjad.string.normalize(
        r"""
        \new Voice
        {
            c'8
            [
            \glissando
            d'8
            \glissando
            e'8
            \glissando
            f'8
            ]
        }
        """
    ), print(abjad.lilypond(voice))

    note = voice[1]
    abjad.mutate.extract(note)

    assert abjad.lilypond(voice) == abjad.string.normalize(
        r"""
        \new Voice
        {
            c'8
            [
            \glissando
            e'8
            \glissando
            f'8
            ]
        }
        """
    ), print(abjad.lilypond(voice))

    assert abjad.wf.wellformed(note)
    assert abjad.wf.wellformed(voice)


def test_mutate_extract_02():
    """
    Extracts multiple notes.
    """

    voice = abjad.Voice("c'8 d'8 e'8 f'8")
    abjad.beam(voice[:])
    abjad.glissando(voice[:])

    assert abjad.lilypond(voice) == abjad.string.normalize(
        r"""
        \new Voice
        {
            c'8
            [
            \glissando
            d'8
            \glissando
            e'8
            \glissando
            f'8
            ]
        }
        """
    ), print(abjad.lilypond(voice))

    notes = voice[:2]
    for note in notes:
        abjad.mutate.extract(note)

    assert abjad.lilypond(voice) == abjad.string.normalize(
        r"""
        \new Voice
        {
            e'8
            \glissando
            f'8
            ]
        }
        """
    ), print(abjad.lilypond(voice))

    for note in notes:
        assert abjad.wf.wellformed(note)

    assert abjad.wf.wellformed(voice, check_overlapping_beams=False)


def test_mutate_extract_03():
    """
    Extracts container.
    """

    voice = abjad.Voice()
    voice.append(abjad.Container("c'8 d'8"))
    voice.append(abjad.Container("e'8 f'8"))
    leaves = abjad.select.leaves(voice)
    abjad.beam(leaves)

    assert abjad.lilypond(voice) == abjad.string.normalize(
        r"""
        \new Voice
        {
            {
                c'8
                [
                d'8
            }
            {
                e'8
                f'8
                ]
            }
        }
        """
    ), print(abjad.lilypond(voice))

    container = voice[0]
    abjad.mutate.extract(container)

    assert abjad.lilypond(voice) == abjad.string.normalize(
        r"""
        \new Voice
        {
            c'8
            [
            d'8
            {
                e'8
                f'8
                ]
            }
        }
        """
    ), print(abjad.lilypond(voice))

    assert not container
    assert abjad.wf.wellformed(voice)


def test_mutate_extract_04():
    """
    Extracts multiple containers.
    """

    voice = abjad.Voice()
    voice.append(abjad.Container("c'8 d'8"))
    voice.append(abjad.Container("e'8 f'8"))
    voice.append(abjad.Container("g'8 a'8"))
    leaves = abjad.select.leaves(voice)
    abjad.beam(leaves)
    abjad.glissando(leaves)

    assert abjad.lilypond(voice) == abjad.string.normalize(
        r"""
        \new Voice
        {
            {
                c'8
                [
                \glissando
                d'8
                \glissando
            }
            {
                e'8
                \glissando
                f'8
                \glissando
            }
            {
                g'8
                \glissando
                a'8
                ]
            }
        }
        """
    ), print(abjad.lilypond(voice))

    containers = voice[:2]
    for container in containers:
        abjad.mutate.extract(container)

    assert abjad.lilypond(voice) == abjad.string.normalize(
        r"""
        \new Voice
        {
            c'8
            [
            \glissando
            d'8
            \glissando
            e'8
            \glissando
            f'8
            \glissando
            {
                g'8
                \glissando
                a'8
                ]
            }
        }
        """
    ), print(abjad.lilypond(voice))

    for container in containers:
        assert not container

    assert abjad.wf.wellformed(voice)



import collections
import copy
import dataclasses
import functools
import types
import typing

from . import enumerate as _enumerate
from . import pitch as _pitch
from . import sequence as _sequence


@dataclasses.dataclass(slots=True)
class PitchClassSegment:
    r"""
    Numbered pitch-class segment.

    ..  container:: example

        >>> segment = abjad.PitchClassSegment([-2, -1.5, 6, 7, -1.5, 7])
        >>> segment
        PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

        >>> lilypond_file = abjad.illustrate(segment)
        >>> abjad.show(lilypond_file) # doctest: +SKIP

        ..  docs::

            >>> voice = lilypond_file["Voice"]
            >>> string = abjad.lilypond(voice)
            >>> print(string)
            \context Voice = "Voice"
            {
                bf'8
                bqf'8
                fs'8
                g'8
                bqf'8
                g'8
                \bar "|."
                \override Score.BarLine.transparent = ##f
            }

    """

    items: typing.Sequence = ()

    def __post_init__(self):
        self.items = tuple(_pitch.NumberedPitchClass(_) for _ in self.items or [])

    def __add__(self, argument) -> "PitchClassSegment":
        """
        Adds ``argument`` to numbered pitch-class segment.
        """
        assert isinstance(argument, type(self)), repr(argument)
        items = tuple(self.items) + tuple(argument.items)
        return type(self)(items)

    def __contains__(self, argument) -> bool:
        """
        Is true when numbered pitch-class segment contains ``argument``.
        """
        return argument in self.items

    def __getitem__(self, argument):
        """
        Gets ``argument`` from numbered pitch-class segment.
        """
        item = self.items.__getitem__(argument)
        if isinstance(item, tuple):
            return type(self)(item)
        else:
            return item

    def __iter__(self):
        """
        Iterates numbered pitch-class segment.
        """
        return self.items.__iter__()

    def __len__(self):
        """
        Gets length of numbered pitch-class segment.
        """
        return len(self.items)

    def __mul__(self, n) -> "PitchClassSegment":
        r"""
        Multiplies numbered pitch-class segment by ``n``.

        ..  container:: example

            >>> segment = abjad.PitchClassSegment([-2, -1.5, 6, 7, -1.5, 7])

            >>> 2 * segment
            PitchClassSegment([10, 10.5, 6, 7, 10.5, 7, 10, 10.5, 6, 7, 10.5, 7])

        """
        return type(self)(n * self.items)

    def __repr__(self) -> str:
        """
        Gets repr.
        """
        items = [_.number for _ in self.items]
        return f"{type(self).__name__}({items!r})"

    def __rmul__(self, n) -> "PitchClassSegment":
        r"""
        Multiplies ``n`` by numbered pitch-class segment.

        ..  container:: example

            >>> segment = abjad.PitchClassSegment([-2, -1.5, 6, 7, -1.5, 7])

            >>> segment * 2
            PitchClassSegment([10, 10.5, 6, 7, 10.5, 7, 10, 10.5, 6, 7, 10.5, 7])

        """
        return type(self)(self.items * n)

    def __str__(self) -> str:
        r"""
        Gets string representation of numbered pitch-class segment.

        ..  container::

            >>> segment = abjad.PitchClassSegment([-2, -1.5, 6, 7, -1.5, 7])
            >>> str(segment)
            'PC<10, 10.5, 6, 7, 10.5, 7>'

        """
        string = ", ".join([str(_.number) for _ in self])
        return f"PC<{string}>"

    def invert(self, axis=None) -> "PitchClassSegment":
        r"""
        Inverts numbered pitch-class segment.

        ..  container:: example

            >>> segment = abjad.PitchClassSegment([-2, -1.5, 6, 7, -1.5, 7])

            >>> segment.invert()
            PitchClassSegment([2, 1.5, 6, 5, 1.5, 5])

            >>> segment.invert().invert()
            PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

        """
        items = [_.invert(axis=axis) for _ in self]
        return type(self)(items)

    def multiply(self, n=1) -> "PitchClassSegment":
        r"""
        Multiplies pitch-classes in numbered pitch-class segment by ``n``.

        ..  container:: example

            >>> segment = abjad.PitchClassSegment([-2, -1.5, 6, 7, -1.5, 7])
            >>> segment
            PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

            >>> segment.multiply(n=1)
            PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

            >>> segment.multiply(n=5)
            PitchClassSegment([2, 4.5, 6, 11, 4.5, 11])

            >>> segment.multiply(n=7)
            PitchClassSegment([10, 1.5, 6, 1, 1.5, 1])

            >>> segment.multiply(n=99)
            PitchClassSegment([6, 7.5, 6, 9, 7.5, 9])

        """
        items = [_.multiply(n=n) for _ in self]
        return type(self)(items)

    def retrograde(self) -> "PitchClassSegment":
        r"""
        Gets retrograde of numbered pitch-class segment.

        ..  container:: example

            >>> segment = abjad.PitchClassSegment([-2, -1.5, 6, 7, -1.5, 7])

            >>> segment.retrograde()
            PitchClassSegment([7, 10.5, 7, 6, 10.5, 10])

            >>> segment.retrograde().retrograde()
            PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

        """
        items = tuple(reversed(self.items))
        return type(self)(items)

    def rotate(self, n=0) -> "PitchClassSegment":
        r"""
        Rotates numbered pitch-class segment by index ``n``.

        ..  container:: example

            >>> segment = abjad.PitchClassSegment([-2, -1.5, 6, 7, -1.5, 7])

            >>> segment.rotate(n=1)
            PitchClassSegment([7, 10, 10.5, 6, 7, 10.5])

            >>> segment.rotate(n=-1)
            PitchClassSegment([10.5, 6, 7, 10.5, 7, 10])

            >>> segment.rotate(n=0)
            PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

        """
        items = _sequence.rotate(self.items, n=n)
        return type(self)(items)

    def transpose(self, n=0) -> "PitchClassSegment":
        r"""
        Transposes numbered pitch-class segment by index ``n``.

        ..  container:: example

            >>> segment = abjad.PitchClassSegment([-2, -1.5, 6, 7, -1.5, 7])

            >>> segment.transpose(n=11)
            PitchClassSegment([9, 9.5, 5, 6, 9.5, 6])

        """
        items = [_.transpose(n=n) for _ in self]
        return type(self)(items)


collections.abc.Sequence.register(PitchClassSegment)


class PitchClassSet(frozenset):
    """
    Numbered pitch-class set.

    ..  container:: example

        >>> set_ = abjad.PitchClassSet([-2, -1.5, 6, 7, -1.5, 7])
        >>> set_
        PitchClassSet([6, 7, 10, 10.5])

        >>> str(set_)
        'PC{6, 7, 10, 10.5}'

        >>> abjad.NamedPitch("fs") in set_
        False

        >>> abjad.NamedPitch("f") in set_
        False

        >>> 6 in set_
        False

        >>> 5 in set_
        False

        >>> abjad.NumberedPitchClass(5) in set_
        False

        >>> abjad.NumberedPitchClass(6) in set_
        True

    ..  container:: example

        Classical operators:

        >>> set_1 = abjad.PitchClassSet([0, 2, 4, 5])
        >>> set_2 = abjad.PitchClassSet([4, 5, 9])

        >>> set_1 & set_2
        PitchClassSet([4, 5])

        >>> set_1 | set_2
        PitchClassSet([0, 2, 4, 5, 9])

        >>> set_1 ^ set_2
        PitchClassSet([0, 2, 9])

        >>> set_1 - set_2
        PitchClassSet([0, 2])

        >>> set_2 - set_1
        PitchClassSet([9])

    """

    def __new__(class_, argument=()):
        if isinstance(argument, str):
            argument = argument.split()
        argument = [_pitch.NumberedPitchClass(_) for _ in argument]
        self = frozenset.__new__(class_, argument)
        return self

    @classmethod
    def _wrap_methods(class_, names):
        def closure(name):
            def inner(self, *arguments):
                result = getattr(super(class_, self), name)(*arguments)
                result = PitchClassSet(result)
                return result

            inner.fn_name = name
            setattr(class_, name, inner)

        for name in names:
            closure(name)

    def __repr__(self):
        numbers = [_.number for _ in self]
        numbers.sort()
        return f"{type(self).__name__}({numbers!r})"

    def __str__(self) -> str:
        numbers = [_.number for _ in self]
        numbers.sort()
        string = ", ".join(str(_) for _ in numbers)
        return f"PC{{{string}}}"

    @property
    def cardinality(self):
        return len(self)

    def get_normal_order(self) -> "PitchClassSegment":
        """
        Gets normal order.

        ..  container:: example

            Gets normal order of empty pitch-class set:

            >>> pc_set = abjad.PitchClassSet()
            >>> pc_set.get_normal_order()
            PitchClassSegment([])

        ..  container:: example

            Gets normal order:

            >>> pc_set = abjad.PitchClassSet([0, 1, 10, 11])
            >>> pc_set.get_normal_order()
            PitchClassSegment([10, 11, 0, 1])

        ..  container:: example

            Gets normal order:

            >>> pc_set = abjad.PitchClassSet([2, 8, 9])
            >>> pc_set.get_normal_order()
            PitchClassSegment([8, 9, 2])

        ..  container:: example

            Gets normal order of pitch-class set with degree of symmetry equal
            to 2:

            >>> pc_set = abjad.PitchClassSet([1, 2, 7, 8])
            >>> pc_set.get_normal_order()
            PitchClassSegment([1, 2, 7, 8])

        ..  container:: example

            Gets normal order of pitch-class set with degree of symmetry equal
            to 4:

            >>> pc_set = abjad.PitchClassSet([0, 3, 6, 9])
            >>> pc_set.get_normal_order()
            PitchClassSegment([0, 3, 6, 9])

        """
        if not len(self):
            return PitchClassSegment()
        pitch_classes = list(self)
        pitch_classes.sort()
        candidates = []
        for i in range(self.cardinality):
            candidate_list = [_pitch.NumberedPitch(_) for _ in pitch_classes]
            candidate = _sequence.rotate(candidate_list, n=-i)
            candidates.append(candidate)
        return _get_most_compact_ordering(candidates)

    def get_prime_form(self, transposition_only=False) -> "PitchClassSet":
        """
        Gets prime form.

        ..  container:: example

            Gets prime form of empty pitch-class set:

            >>> pc_set = abjad.PitchClassSet()
            >>> pc_set.get_prime_form()
            PitchClassSet([])

            >>> pc_set = abjad.PitchClassSet()
            >>> pc_set.get_prime_form(transposition_only=True)
            PitchClassSet([])

        ..  container:: example

            Gets prime form:

            >>> pc_set = abjad.PitchClassSet([0, 1, 10, 11])
            >>> pc_set.get_prime_form()
            PitchClassSet([0, 1, 2, 3])

            >>> pc_set = abjad.PitchClassSet([0, 1, 10, 11])
            >>> pc_set.get_prime_form(transposition_only=True)
            PitchClassSet([0, 1, 2, 3])

        ..  container:: example

            Gets prime form:

            >>> pc_set = abjad.PitchClassSet([2, 8, 9])
            >>> pc_set.get_prime_form()
            PitchClassSet([0, 1, 6])

            >>> pc_set = abjad.PitchClassSet([2, 8, 9])
            >>> pc_set.get_prime_form(transposition_only=True)
            PitchClassSet([0, 1, 6])

        ..  container:: example

            Gets prime form of pitch-class set with degree of symmetry equal to
            2:

            >>> pc_set = abjad.PitchClassSet([1, 2, 7, 8])
            >>> pc_set.get_prime_form()
            PitchClassSet([0, 1, 6, 7])

            >>> pc_set = abjad.PitchClassSet([1, 2, 7, 8])
            >>> pc_set.get_prime_form(transposition_only=True)
            PitchClassSet([0, 1, 6, 7])

        ..  container:: example

            Gets prime form of pitch-class set with degree of symmetry equal to
            4:

            >>> pc_set = abjad.PitchClassSet([0, 3, 6, 9])
            >>> pc_set.get_prime_form()
            PitchClassSet([0, 3, 6, 9])

            >>> pc_set = abjad.PitchClassSet([0, 3, 6, 9])
            >>> pc_set.get_prime_form(transposition_only=True)
            PitchClassSet([0, 3, 6, 9])

        ..  container:: example

            Gets prime form of pitch-class that is not inversion-equivalent:

            >>> pc_set = abjad.PitchClassSet([0, 4, 6, 7])
            >>> pc_set.get_prime_form()
            PitchClassSet([0, 1, 3, 7])

            >>> pc_set = abjad.PitchClassSet([0, 4, 6, 7])
            >>> pc_set.get_prime_form(transposition_only=True)
            PitchClassSet([0, 4, 6, 7])

        ..  container:: example

            Gets prime form of inversionally nonequivalent pitch-class set:

            >>> pc_set = abjad.PitchClassSet([0, 4, 7])
            >>> pc_set.get_prime_form()
            PitchClassSet([0, 3, 7])

            >>> pc_set = abjad.PitchClassSet([0, 4, 7])
            >>> pc_set.get_prime_form(transposition_only=True)
            PitchClassSet([0, 4, 7])

        ..  container:: example

            REGRESSION:

            >>> pc_set = abjad.PitchClassSet([0, 1, 2, 5, 8, 9])
            >>> pc_set.get_prime_form()
            PitchClassSet([0, 1, 2, 5, 6, 9])

            REGRESSION:

            >>> pc_set = abjad.PitchClassSet([0, 1, 2, 3, 6, 7])
            >>> pc_set.get_prime_form()
            PitchClassSet([0, 1, 2, 3, 6, 7])

        Returns new pitch-class set.
        """

        def _transpose_to_zero(segment):
            numbers = [_.number for _ in segment]
            first_number = segment[0].number
            numbers = [pc.number - first_number for pc in segment]
            pcs = [_ % 12 for _ in numbers]
            return type(segment)(pcs)

        if not len(self):
            return copy.copy(self)
        normal_order = self.get_normal_order()
        if not transposition_only:
            normal_orders = [normal_order]
            inversion = self.invert()
            normal_order = inversion.get_normal_order()
            normal_orders.append(normal_order)
            normal_orders = [_transpose_to_zero(_) for _ in normal_orders]
            assert len(normal_orders) == 2
            for left_pc, right_pc in zip(*normal_orders):
                if left_pc == right_pc:
                    continue
                if left_pc < right_pc:
                    normal_order = normal_orders[0]
                    break
                if right_pc < left_pc:
                    normal_order = normal_orders[-1]
                    break
        pcs = [_.number for _ in normal_order]
        first_pc = pcs[0]
        pcs = [pc - first_pc for pc in pcs]
        prime_form = type(self)(pcs)
        return prime_form

    def invert(self, axis=None) -> "PitchClassSet":
        """
        Inverts pitch-class set.

        ..  container:: example

            >>> abjad.PitchClassSet([-2, -1.5, 6, 7, -1.5, 7]).invert()
            PitchClassSet([1.5, 2, 5, 6])

        """
        return type(self)([_.invert(axis=axis) for _ in self])

    def multiply(self, n) -> "PitchClassSet":
        """
        Multiplies pitch-class set by ``n``.

        ..  container:: example

            >>> abjad.PitchClassSet([-2, -1.5, 6, 7, -1.5, 7]).multiply(5)
            PitchClassSet([2, 4.5, 6, 11])

        """
        return type(self)(_.multiply(n) for _ in self)

    def transpose(self, n=0) -> "PitchClassSet":
        """
        Transposes all pitch-classes in pitch-class set by index ``n``.

        ..  container:: example

            >>> set_ = abjad.PitchClassSet([-2, -1.5, 6, 7, -1.5, 7])
            >>> for n in range(12):
            ...     print(n, set_.transpose(n))
            ...
            0 PC{6, 7, 10, 10.5}
            1 PC{7, 8, 11, 11.5}
            2 PC{0, 0.5, 8, 9}
            3 PC{1, 1.5, 9, 10}
            4 PC{2, 2.5, 10, 11}
            5 PC{0, 3, 3.5, 11}
            6 PC{0, 1, 4, 4.5}
            7 PC{1, 2, 5, 5.5}
            8 PC{2, 3, 6, 6.5}
            9 PC{3, 4, 7, 7.5}
            10 PC{4, 5, 8, 8.5}
            11 PC{5, 6, 9, 9.5}

        """
        return type(self)(_ + n for _ in self)


collections.abc.Sequence.register(PitchClassSet)


PitchClassSet._wrap_methods(
    [
        "__and__",
        "__or__",
        "__sub__",
        "__xor__",
        "difference",
        "intersection",
        "symmetric_difference",
        "union",
    ]
)


@functools.total_ordering
class PitchRange:
    r"""
    Pitch range.

    ..  container:: example

        Pitches from C3 to C7, inclusive:

        >>> pitch_range = abjad.PitchRange("[C3, C7]")
        >>> lilypond_file = abjad.illustrate(pitch_range)
        >>> abjad.show(lilypond_file) # doctest: +SKIP

        ..  docs::

            >>> score = lilypond_file["Score"]
            >>> string = abjad.lilypond(score)
            >>> print(string)
            \context Score = "Score"
            \with
            {
                \override BarLine.stencil = ##f
                \override Glissando.thickness = 2
                \override SpanBar.stencil = ##f
                \override TimeSignature.stencil = ##f
            }
            <<
                \context PianoStaff = "Piano_Staff"
                <<
                    \context Staff = "Treble_Staff"
                    {
                        \context Voice = "Treble_Voice"
                        {
                            \clef "treble"
                            s1 * 1/4
                            s1 * 1/4
                        }
                    }
                    \context Staff = "Bass_Staff"
                    {
                        \context Voice = "Bass_Voice"
                        {
                            \clef "bass"
                            c1 * 1/4
                            \glissando
                            \change Staff = Treble_Staff
                            c''''1 * 1/4
                        }
                    }
                >>
            >>

    """

    __slots__ = (
        "_close_bracket",
        "_open_bracket",
        "_range_string",
        "_start_pitch",
        "_stop_pitch",
    )

    def __init__(self, range_string="[A0, C8]"):
        if isinstance(range_string, type(self)):
            range_string = range_string.range_string
        assert isinstance(range_string, str), repr(range_string)
        namespace = self._parse_range_string(range_string)
        assert isinstance(namespace.start_pitch, _pitch.NamedPitch | type(None))
        assert isinstance(namespace.stop_pitch, _pitch.NamedPitch | type(None))
        self._close_bracket = namespace.close_bracket
        self._open_bracket = namespace.open_bracket
        self._range_string = namespace.range_string
        self._start_pitch = namespace.start_pitch
        self._stop_pitch = namespace.stop_pitch

    def __contains__(self, argument) -> bool:
        """
        Is true when pitch range contains ``argument``.

        ..  container:: example

            Closed / closed range:

            >>> range_ = abjad.PitchRange("[A0, C8]")

            >>> "C0" in range_
            False

            >>> "A0" in range_
            True

            >>> "C4" in range_
            True

            >>> "C8" in range_
            True

            >>> "C12" in range_
            False

        ..  container:: example

            Closed / open range:

            >>> range_ = abjad.PitchRange("[A0, C8)")

            >>> "C0" in range_
            False

            >>> "A0" in range_
            True

            >>> "C4" in range_
            True

            >>> "C8" in range_
            False

            >>> "C12" in range_
            False

        ..  container:: example

            Closed / infinite range:

            >>> range_ = abjad.PitchRange("[A0, +inf]")

            >>> "C0" in range_
            False

            >>> "A0" in range_
            True

            >>> "C4" in range_
            True

            >>> "C8" in range_
            True

            >>> "C12" in range_
            True

        ..  container:: example

            Open / closed range:

            >>> range_ = abjad.PitchRange("(A0, C8]")

            >>> "C0" in range_
            False

            >>> "A0" in range_
            False

            >>> "C4" in range_
            True

            >>> "C8" in range_
            True

            >>> "C12" in range_
            False

        ..  container:: example

            Open / open range:

            >>> range_ = abjad.PitchRange("(A0, C8)")

            >>> "C0" in range_
            False

            >>> "A0" in range_
            False

            >>> "C4" in range_
            True

            >>> "C8" in range_
            False

            >>> "C12" in range_
            False

        ..  container:: example

            Infinite / closed range:

            >>> range_ = abjad.PitchRange("[-inf, C8]")

            >>> "C0" in range_
            True

            >>> "A0" in range_
            True

            >>> "C4" in range_
            True

            >>> "C8" in range_
            True

            >>> "C12" in range_
            False

        ..  container:: example

            Infinite / open range:

            >>> range_ = abjad.PitchRange("[-inf, C8)")

            >>> "C0" in range_
            True

            >>> "A0" in range_
            True

            >>> "C4" in range_
            True

            >>> "C8" in range_
            False

            >>> "C12" in range_
            False

        ..  container:: example

            Infinite / infinite range:

            >>> range_ = abjad.PitchRange("[-inf, +inf]")

            >>> "C0" in range_
            True

            >>> "A0" in range_
            True

            >>> "C4" in range_
            True

            >>> "C8" in range_
            True

            >>> "C12" in range_
            True

        ..  container:: example

            Raises exception when argument is not a named pitch or string:

            >>> -99 in range_
            Traceback (most recent call last):
                ...
            Exception: must be named pitch or string (not -99).

        """
        if isinstance(argument, str | _pitch.NamedPitch):
            pitch = _pitch.NamedPitch(argument)
            if self.start_pitch is None:
                start_pitch = _pitch.NamedPitch(-1000)
            else:
                start_pitch = _pitch.NamedPitch(self.start_pitch)
            if self.stop_pitch is None:
                stop_pitch = _pitch.NamedPitch(1000)
            else:
                stop_pitch = _pitch.NamedPitch(self.stop_pitch)
        else:
            raise Exception(f"must be named pitch or string (not {argument!r}).")
        if self._open_bracket == "[":
            if self._close_bracket == "]":
                return start_pitch <= pitch <= stop_pitch
            else:
                return start_pitch <= pitch < stop_pitch
        else:
            if self._close_bracket == "]":
                return start_pitch < pitch <= stop_pitch
            else:
                return start_pitch < pitch < stop_pitch

    def __eq__(self, argument) -> bool:
        """
        Is true when ``argument`` is a pitch range with ``range_string`` equal to this
        pitch range.

        ..  container:: example

            >>> range_1 = abjad.PitchRange("[A0, C4]")
            >>> range_2 = abjad.PitchRange("[A0, C4]")
            >>> range_3 = abjad.PitchRange("[A0, C8]")

            >>> range_1 == range_1
            True
            >>> range_1 == range_2
            True
            >>> range_1 == range_3
            False

            >>> range_2 == range_1
            True
            >>> range_2 == range_2
            True
            >>> range_2 == range_3
            False

            >>> range_3 == range_1
            False
            >>> range_3 == range_2
            False
            >>> range_3 == range_3
            True

        """
        if isinstance(argument, type(self)):
            return self.range_string == argument.range_string
        return False

    def __hash__(self) -> int:
        """
        Hashes pitch range.
        """
        return hash(repr(self))

    def __lt__(self, argument) -> bool:
        """
        Is true when start pitch of this pitch-range is less than start pitch of
        ``argument`` pitch range.

        ..  container:: example

            >>> range_1 = abjad.PitchRange("[A0, C4]")
            >>> range_2 = abjad.PitchRange("[A0, C4]")
            >>> range_3 = abjad.PitchRange("[A0, C8]")

            >>> range_1 < range_1
            False
            >>> range_1 < range_2
            False
            >>> range_1 < range_3
            True

            >>> range_2 < range_1
            False
            >>> range_2 < range_2
            False
            >>> range_2 < range_3
            True

            >>> range_3 < range_1
            False
            >>> range_3 < range_2
            False
            >>> range_3 < range_3
            False

        """
        assert isinstance(argument, type(self)), repr(argument)
        if self.start_pitch is None:
            self_start_pitch = _pitch.NamedPitch(-1000)
        else:
            self_start_pitch = _pitch.NamedPitch(self.start_pitch)
        if self.stop_pitch is None:
            self_stop_pitch = _pitch.NamedPitch(1000)
        else:
            self_stop_pitch = _pitch.NamedPitch(self.stop_pitch)
        if argument.start_pitch is None:
            argument_start_pitch = _pitch.NamedPitch(-1000)
        else:
            argument_start_pitch = _pitch.NamedPitch(argument.start_pitch)
        if argument.stop_pitch is None:
            argument_stop_pitch = _pitch.NamedPitch(1000)
        else:
            argument_stop_pitch = _pitch.NamedPitch(argument.stop_pitch)
        if self_start_pitch == argument_start_pitch:
            return self_stop_pitch < argument_stop_pitch
        return self_start_pitch < argument_start_pitch

    def __repr__(self) -> str:
        """
        Gets pitch range interpreter representation.
        """
        return f"{type(self).__name__}(range_string={self.range_string!r})"

    def _parse_range_string(self, range_string):
        assert isinstance(range_string, str), repr(range_string)
        range_string = range_string.replace("-inf", "-1000")
        range_string = range_string.replace("+inf", "1000")
        match = _pitch._range_string_regex.match(range_string)
        if match is None:
            raise ValueError(f"can not parse range string: {range_string!r}")
        group_dict = match.groupdict()
        open_bracket = group_dict["open_bracket"]
        start_pitch_string = group_dict["start_pitch"]
        stop_pitch_string = group_dict["stop_pitch"]
        close_bracket = group_dict["close_bracket"]
        if start_pitch_string == "-1000":
            start_pitch = None
            start_pitch_repr = "-inf"
        elif start_pitch_string.isnumeric() or start_pitch_string.startswith("-"):
            raise Exception(f"named pitches only (not {range_string!r}).")
        else:
            start_pitch = _pitch.NamedPitch(start_pitch_string)
            start_pitch_repr = start_pitch.get_name(locale="us")

        if stop_pitch_string == "1000":
            stop_pitch = None
            stop_pitch_repr = "+inf"
        elif stop_pitch_string.isnumeric() or stop_pitch_string.startswith("-"):
            raise Exception(f"named pitches only (not {range_string!r}).")
        else:
            stop_pitch = _pitch.NamedPitch(stop_pitch_string)
            stop_pitch_repr = stop_pitch.get_name(locale="us")
        start, stop = open_bracket, close_bracket
        normalized_range_string = f"{start}{start_pitch_repr}, {stop_pitch_repr}{stop}"
        return types.SimpleNamespace(
            close_bracket=close_bracket,
            open_bracket=open_bracket,
            range_string=normalized_range_string,
            start_pitch=start_pitch,
            stop_pitch=stop_pitch,
        )

    @property
    def range_string(self) -> str:
        """
        Gets range string of pitch range.

        ..  container:: example

            >>> abjad.PitchRange("[C3, C7]").range_string
            '[C3, C7]'

            >>> abjad.PitchRange("[-inf, C7]").range_string
            '[-inf, C7]'

            >>> abjad.PitchRange("[C3, +inf]").range_string
            '[C3, +inf]'

            >>> abjad.PitchRange("[-inf, +inf]").range_string
            '[-inf, +inf]'

        """
        return self._range_string

    @property
    def start_pitch(self) -> _pitch.NamedPitch | None:
        """
        Start pitch of pitch range.

        ..  container:: example

            >>> abjad.PitchRange("[C3, C7]").start_pitch
            NamedPitch('c')

            >>> abjad.PitchRange("[-inf, C7]").start_pitch is None
            True

        """
        return self._start_pitch

    @property
    def stop_pitch(self) -> _pitch.NamedPitch | None:
        """
        Stop pitch of pitch range.

        ..  container:: example

            >>> abjad.PitchRange("[C3, C7]").stop_pitch
            NamedPitch("c''''")

            >>> abjad.PitchRange("[C8, +inf]").stop_pitch is None
            True

        """
        return self._stop_pitch

    @staticmethod
    def from_octave(octave) -> "PitchRange":
        """
        Initializes pitch range from octave.

        ..  container:: example

            >>> abjad.PitchRange.from_octave(5)
            PitchRange(range_string='[C5, C6)')

        """
        octave = _pitch.Octave(octave)
        return PitchRange(f"[C{octave.number}, C{octave.number + 1})")

    def voice_pitch_class(self, pitch_class):
        """
        Voices ``pitch_class``.

        ..  container:: example

            Voices C three times:

            >>> pitch_range = abjad.PitchRange("[C4, C6]")
            >>> pitch_range.voice_pitch_class("c")
            (NamedPitch("c'"), NamedPitch("c''"), NamedPitch("c'''"))

            Voices B two times:

            >>> pitch_range = abjad.PitchRange("[C4, C6]")
            >>> pitch_range.voice_pitch_class("b")
            (NamedPitch("b'"), NamedPitch("b''"))

            Returns empty because B can not voice:

            >>> pitch_range = abjad.PitchRange("[C4, A4)")
            >>> pitch_range.voice_pitch_class("b")
            ()

        """
        named_pitch_class = _pitch.NamedPitchClass(pitch_class)
        pair = (named_pitch_class.name, self.start_pitch.octave.number)
        named_pitch = _pitch.NamedPitch(pair)
        result = []
        while named_pitch <= self.stop_pitch:
            if named_pitch in self:
                result.append(named_pitch)
            named_pitch += 12
        return tuple(result)


@dataclasses.dataclass(slots=True)
class PitchSegment:
    r"""
    Named pitch segment.

    ..  container:: example

        >>> segment = abjad.PitchSegment([-2, -1.5, 6, 7, -1.5, 7])

        >>> str(segment)
        '<-2, -1.5, 6, 7, -1.5, 7>'

        >>> lilypond_file = abjad.illustrate(segment)
        >>> abjad.show(lilypond_file) # doctest: +SKIP

        ..  docs::

            >>> staff_group = lilypond_file["Piano_Staff"]
            >>> string = abjad.lilypond(staff_group)
            >>> print(string)
            \context PianoStaff = "Piano_Staff"
            <<
                \context Staff = "Treble_Staff"
                {
                    \context Voice = "Treble_Voice"
                    {
                        \clef "treble"
                        r1 * 1/8
                        r1 * 1/8
                        fs'1 * 1/8
                        g'1 * 1/8
                        r1 * 1/8
                        g'1 * 1/8
                    }
                }
                \context Staff = "Bass_Staff"
                {
                    \context Voice = "Bass_Voice"
                    {
                        \clef "bass"
                        bf1 * 1/8
                        bqf1 * 1/8
                        r1 * 1/8
                        r1 * 1/8
                        bqf1 * 1/8
                        r1 * 1/8
                    }
                }
            >>

    """

    items: typing.Sequence = ()

    def __post_init__(self):
        self.items = tuple(_pitch.NumberedPitch(_) for _ in self.items or [])

    def __contains__(self, argument) -> bool:
        """
        Is true when named pitch segment contains ``argument``.

        ..  container:: example

            >>> segment = abjad.PitchSegment([-2, -1.5, 6, 7, -1.5, 7])

            >>> abjad.NamedPitch("fs'") in segment
            False

            >>> abjad.NamedPitch(6) in segment
            False

            >>> abjad.NamedPitch("fs") in segment
            False

            >>> 6 in segment
            True

            >>> abjad.NumberedPitch(6) in segment
            True

            >>> abjad.NumberedPitch("fs'") in segment
            True

            >>> abjad.NamedPitch("f") in segment
            False

            >>> 5 in segment
            False

        """
        return argument in self.items

    def __getitem__(self, argument):
        """
        Gets item at ``argument`` in named pitch segment.
        """
        return self.items.__getitem__(argument)

    def __iter__(self):
        """
        Iterates named pitch segment.
        """
        return self.items.__iter__()

    def __len__(self):
        """
        Gets length of named pitch segment.
        """
        return len(self.items)

    def __repr__(self):
        """
        Gets repr.
        """
        numbers = [_.number for _ in self.items]
        return f"{type(self).__name__}({numbers})"

    def __str__(self) -> str:
        """
        Gets string.

        ..  container:: example

            >>> segment = abjad.PitchSegment([-2, -1.5, 6, 7, -1.5, 7])

            >>> str(segment)
            '<-2, -1.5, 6, 7, -1.5, 7>'

        """
        string = ", ".join([str(_.number) for _ in self])
        return f"<{string}>"

    def invert(self, axis=None) -> "PitchSegment":
        r"""
        Inverts named pitch segment about ``axis``.

        ..  container:: example

            >>> segment = abjad.PitchSegment([-2, -1.5, 6, 7, -1.5, 7])

            >>> lilypond_file = abjad.illustrate(segment)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> staff_group = lilypond_file["Piano_Staff"]
                >>> string = abjad.lilypond(staff_group)
                >>> print(string)
                \context PianoStaff = "Piano_Staff"
                <<
                    \context Staff = "Treble_Staff"
                    {
                        \context Voice = "Treble_Voice"
                        {
                            \clef "treble"
                            r1 * 1/8
                            r1 * 1/8
                            fs'1 * 1/8
                            g'1 * 1/8
                            r1 * 1/8
                            g'1 * 1/8
                        }
                    }
                    \context Staff = "Bass_Staff"
                    {
                        \context Voice = "Bass_Voice"
                        {
                            \clef "bass"
                            bf1 * 1/8
                            bqf1 * 1/8
                            r1 * 1/8
                            r1 * 1/8
                            bqf1 * 1/8
                            r1 * 1/8
                        }
                    }
                >>

            >>> segment = segment.invert(axis=0)

            >>> str(segment)
            '<2, 1.5, -6, -7, 1.5, -7>'

            >>> lilypond_file = abjad.illustrate(segment)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> staff_group = lilypond_file["Piano_Staff"]
                >>> string = abjad.lilypond(staff_group)
                >>> print(string)
                \context PianoStaff = "Piano_Staff"
                <<
                    \context Staff = "Treble_Staff"
                    {
                        \context Voice = "Treble_Voice"
                        {
                            \clef "treble"
                            d'1 * 1/8
                            dqf'1 * 1/8
                            r1 * 1/8
                            r1 * 1/8
                            dqf'1 * 1/8
                            r1 * 1/8
                        }
                    }
                    \context Staff = "Bass_Staff"
                    {
                        \context Voice = "Bass_Voice"
                        {
                            \clef "bass"
                            r1 * 1/8
                            r1 * 1/8
                            fs1 * 1/8
                            f1 * 1/8
                            r1 * 1/8
                            f1 * 1/8
                        }
                    }
                >>

        """
        items = [_.invert(axis=axis) for _ in self]
        return dataclasses.replace(self, items=items)

    def multiply(self, n=1) -> "PitchSegment":
        r"""
        Multiplies pitch segment by index ``n``.

        ..  container:: example

            >>> segment = abjad.PitchSegment([-2, -1.5, 6, 7, -1.5, 7])

            >>> lilypond_file = abjad.illustrate(segment)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> staff_group = lilypond_file["Piano_Staff"]
                >>> string = abjad.lilypond(staff_group)
                >>> print(string)
                \context PianoStaff = "Piano_Staff"
                <<
                    \context Staff = "Treble_Staff"
                    {
                        \context Voice = "Treble_Voice"
                        {
                            \clef "treble"
                            r1 * 1/8
                            r1 * 1/8
                            fs'1 * 1/8
                            g'1 * 1/8
                            r1 * 1/8
                            g'1 * 1/8
                        }
                    }
                    \context Staff = "Bass_Staff"
                    {
                        \context Voice = "Bass_Voice"
                        {
                            \clef "bass"
                            bf1 * 1/8
                            bqf1 * 1/8
                            r1 * 1/8
                            r1 * 1/8
                            bqf1 * 1/8
                            r1 * 1/8
                        }
                    }
                >>

            >>> segment = segment.multiply(n=3)

            >>> str(segment)
            '<-6, -4.5, 18, 21, -4.5, 21>'

            >>> lilypond_file = abjad.illustrate(segment)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> staff_group = lilypond_file["Piano_Staff"]
                >>> string = abjad.lilypond(staff_group)
                >>> print(string)
                \context PianoStaff = "Piano_Staff"
                <<
                    \context Staff = "Treble_Staff"
                    {
                        \context Voice = "Treble_Voice"
                        {
                            \clef "treble"
                            r1 * 1/8
                            r1 * 1/8
                            fs''1 * 1/8
                            a''1 * 1/8
                            r1 * 1/8
                            a''1 * 1/8
                        }
                    }
                    \context Staff = "Bass_Staff"
                    {
                        \context Voice = "Bass_Voice"
                        {
                            \clef "bass"
                            fs1 * 1/8
                            gqs1 * 1/8
                            r1 * 1/8
                            r1 * 1/8
                            gqs1 * 1/8
                            r1 * 1/8
                        }
                    }
                >>

        """
        items = [_.multiply(n=n) for _ in self]
        return dataclasses.replace(self, items=items)

    def retrograde(self) -> "PitchSegment":
        r"""
        Retrograde of pitch segment.

        ..  container:: example

            >>> segment = abjad.PitchSegment([-2, -1.5, 6, 7, -1.5, 7])

            >>> lilypond_file = abjad.illustrate(segment)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> staff_group = lilypond_file["Piano_Staff"]
                >>> string = abjad.lilypond(staff_group)
                >>> print(string)
                \context PianoStaff = "Piano_Staff"
                <<
                    \context Staff = "Treble_Staff"
                    {
                        \context Voice = "Treble_Voice"
                        {
                            \clef "treble"
                            r1 * 1/8
                            r1 * 1/8
                            fs'1 * 1/8
                            g'1 * 1/8
                            r1 * 1/8
                            g'1 * 1/8
                        }
                    }
                    \context Staff = "Bass_Staff"
                    {
                        \context Voice = "Bass_Voice"
                        {
                            \clef "bass"
                            bf1 * 1/8
                            bqf1 * 1/8
                            r1 * 1/8
                            r1 * 1/8
                            bqf1 * 1/8
                            r1 * 1/8
                        }
                    }
                >>

            >>> segment = segment.retrograde()

            >>> str(segment)
            '<7, -1.5, 7, 6, -1.5, -2>'

            >>> lilypond_file = abjad.illustrate(segment)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> staff_group = lilypond_file["Piano_Staff"]
                >>> string = abjad.lilypond(staff_group)
                >>> print(string)
                \context PianoStaff = "Piano_Staff"
                <<
                    \context Staff = "Treble_Staff"
                    {
                        \context Voice = "Treble_Voice"
                        {
                            \clef "treble"
                            g'1 * 1/8
                            r1 * 1/8
                            g'1 * 1/8
                            fs'1 * 1/8
                            r1 * 1/8
                            r1 * 1/8
                        }
                    }
                    \context Staff = "Bass_Staff"
                    {
                        \context Voice = "Bass_Voice"
                        {
                            \clef "bass"
                            r1 * 1/8
                            bqf1 * 1/8
                            r1 * 1/8
                            r1 * 1/8
                            bqf1 * 1/8
                            bf1 * 1/8
                        }
                    }
                >>

        """
        return dataclasses.replace(self, items=list(reversed(self)))

    def rotate(self, n=0) -> "PitchSegment":
        r"""
        Rotates pitch segment by index ``n``.

        ..  container:: example

            >>> segment = abjad.PitchSegment([-2, -1.5, 6, 7, -1.5, 7])

            >>> lilypond_file = abjad.illustrate(segment)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> staff_group = lilypond_file["Piano_Staff"]
                >>> string = abjad.lilypond(staff_group)
                >>> print(string)
                \context PianoStaff = "Piano_Staff"
                <<
                    \context Staff = "Treble_Staff"
                    {
                        \context Voice = "Treble_Voice"
                        {
                            \clef "treble"
                            r1 * 1/8
                            r1 * 1/8
                            fs'1 * 1/8
                            g'1 * 1/8
                            r1 * 1/8
                            g'1 * 1/8
                        }
                    }
                    \context Staff = "Bass_Staff"
                    {
                        \context Voice = "Bass_Voice"
                        {
                            \clef "bass"
                            bf1 * 1/8
                            bqf1 * 1/8
                            r1 * 1/8
                            r1 * 1/8
                            bqf1 * 1/8
                            r1 * 1/8
                        }
                    }
                >>

            >>> segment = segment.rotate(n=1)

            >>> str(segment)
            '<7, -2, -1.5, 6, 7, -1.5>'

            >>> lilypond_file = abjad.illustrate(segment)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> staff_group = lilypond_file["Piano_Staff"]
                >>> string = abjad.lilypond(staff_group)
                >>> print(string)
                \context PianoStaff = "Piano_Staff"
                <<
                    \context Staff = "Treble_Staff"
                    {
                        \context Voice = "Treble_Voice"
                        {
                            \clef "treble"
                            g'1 * 1/8
                            r1 * 1/8
                            r1 * 1/8
                            fs'1 * 1/8
                            g'1 * 1/8
                            r1 * 1/8
                        }
                    }
                    \context Staff = "Bass_Staff"
                    {
                        \context Voice = "Bass_Voice"
                        {
                            \clef "bass"
                            r1 * 1/8
                            bf1 * 1/8
                            bqf1 * 1/8
                            r1 * 1/8
                            r1 * 1/8
                            bqf1 * 1/8
                        }
                    }
                >>

        """
        rotated_pitches = _sequence.rotate(self.items, n=n)
        new_segment = dataclasses.replace(self, items=rotated_pitches)
        return new_segment

    def transpose(self, n=0) -> "PitchSegment":
        r"""
        Transposes pitch segment by index ``n``.

        ..  container:: example

            >>> segment = abjad.PitchSegment([-2, -1.5, 6, 7, -1.5, 7])

            >>> lilypond_file = abjad.illustrate(segment)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> staff_group = lilypond_file["Piano_Staff"]
                >>> string = abjad.lilypond(staff_group)
                >>> print(string)
                \context PianoStaff = "Piano_Staff"
                <<
                    \context Staff = "Treble_Staff"
                    {
                        \context Voice = "Treble_Voice"
                        {
                            \clef "treble"
                            r1 * 1/8
                            r1 * 1/8
                            fs'1 * 1/8
                            g'1 * 1/8
                            r1 * 1/8
                            g'1 * 1/8
                        }
                    }
                    \context Staff = "Bass_Staff"
                    {
                        \context Voice = "Bass_Voice"
                        {
                            \clef "bass"
                            bf1 * 1/8
                            bqf1 * 1/8
                            r1 * 1/8
                            r1 * 1/8
                            bqf1 * 1/8
                            r1 * 1/8
                        }
                    }
                >>

            >>> segment = segment.transpose(n=11)

            >>> str(segment)
            '<9, 9.5, 17, 18, 9.5, 18>'

            >>> lilypond_file = abjad.illustrate(segment)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> staff_group = lilypond_file["Piano_Staff"]
                >>> string = abjad.lilypond(staff_group)
                >>> print(string)
                \context PianoStaff = "Piano_Staff"
                <<
                    \context Staff = "Treble_Staff"
                    {
                        \context Voice = "Treble_Voice"
                        {
                            \clef "treble"
                            a'1 * 1/8
                            aqs'1 * 1/8
                            f''1 * 1/8
                            fs''1 * 1/8
                            aqs'1 * 1/8
                            fs''1 * 1/8
                        }
                    }
                    \context Staff = "Bass_Staff"
                    {
                        \context Voice = "Bass_Voice"
                        {
                            \clef "bass"
                            r1 * 1/8
                            r1 * 1/8
                            r1 * 1/8
                            r1 * 1/8
                            r1 * 1/8
                            r1 * 1/8
                        }
                    }
                >>

        """
        if isinstance(n, int | float):
            interval = _pitch.NumberedInterval(n)
        else:
            interval = n
        assert isinstance(interval, _pitch.NumberedInterval), repr(interval)
        items = [_.transpose(n=interval) for _ in self]
        return type(self)(items)


collections.abc.Sequence.register(PitchSegment)


class PitchSet(frozenset):
    """
    Numbered pitch set.

    ..  container:: example

        >>> set_1 = abjad.PitchSet([-26, -2.5, 6, 7, -1.5, 7])
        >>> set_2 = abjad.PitchSet([-1.5, 7, 9, 12])

        >>> set_1
        PitchSet([-26, -2.5, -1.5, 6, 7])

        >>> set_2
        PitchSet([-1.5, 7, 9, 12])

        >>> str(set_1)
        '{-26, -2.5, -1.5, 6, 7}'

        >>> str(set_2)
        '{-1.5, 7, 9, 12}'

        >>> -1.5 in set_1
        False

        >>> abjad.NumberedPitch(-1.5) in set_1
        True

        >>> abjad.NumberedPitch("aqs") in set_2
        False

        >>> len(set_1)
        5

        >>> len(set_2)
        4

    ..  container:: example

        Classical operators:

        >>> set_1 = abjad.PitchSet([-26, -2.5, 6, 7, -1.5, 7])
        >>> set_2 = abjad.PitchSet([-1.5, 7, 9, 12])

        >>> set_1 & set_2
        PitchSet([-1.5, 7])

        >>> set_2 & set_1
        PitchSet([-1.5, 7])

        >>> set_1.intersection(set_2)
        PitchSet([-1.5, 7])

        >>> set_2.intersection(set_1)
        PitchSet([-1.5, 7])

        >>> set_1 | set_2
        PitchSet([-26, -2.5, -1.5, 6, 7, 9, 12])

        >>> set_2 | set_1
        PitchSet([-26, -2.5, -1.5, 6, 7, 9, 12])

        >>> set_1.union(set_2)
        PitchSet([-26, -2.5, -1.5, 6, 7, 9, 12])

        >>> set_2.union(set_1)
        PitchSet([-26, -2.5, -1.5, 6, 7, 9, 12])

        >>> set_1 - set_2
        PitchSet([-26, -2.5, 6])

        >>> set_1.difference(set_2)
        PitchSet([-26, -2.5, 6])

        >>> set_2 - set_1
        PitchSet([9, 12])

        >>> set_2.difference(set_1)
        PitchSet([9, 12])

        >>> set_1 ^ set_2
        PitchSet([-26, -2.5, 6, 9, 12])

        >>> set_2 ^ set_1
        PitchSet([-26, -2.5, 6, 9, 12])

        >>> set_1.symmetric_difference(set_2)
        PitchSet([-26, -2.5, 6, 9, 12])

        >>> set_2.symmetric_difference(set_1)
        PitchSet([-26, -2.5, 6, 9, 12])

    ..  container:: example

        >>> set_1 = abjad.PitchSet([-6, -5, -3, -1])
        >>> set_2 = abjad.PitchSet([-3, -1])
        >>> set_3 = abjad.PitchSet([1, 3, 4])

        Predicates:

        >>> set_1.isdisjoint(set_3)
        True

        >>> set_2.issubset(set_1)
        True

        >>> set_1.issuperset(set_2)
        True

    ..  container:: example

        >>> set_1 = abjad.PitchSet([-26, -2.5, 6, 7, -1.5, 7])
        >>> set_2 = abjad.PitchSet([-1.5, 7, 9, 12])

        Comparison:

        >>> set_1 == set_1
        True

        >>> set_1 == set_2
        False

        >>> set_1 < set_1
        False

        >>> set_1 < set_2
        False

        >>> set_1 <= set_1
        True

        >>> set_1 <= set_2
        False

        >>> set_1 > set_1
        False

        >>> set_1 > set_2
        False

        >>> set_1 >= set_1
        True

        >>> set_1 >= set_2
        False

    ..  container:: example

        Copy:

        >>> import copy
        >>> set_1_copy = copy.copy(set_1)
        >>> set_1_copy
        PitchSet([-26, -2.5, -1.5, 6, 7])

        >>> set_1 == set_1_copy
        True

    ..  container:: example

        Empty set:

        >>> set_ = abjad.PitchSet()
        >>> set_
        PitchSet()

        >>> str(set_)
        '{}'

        >>> len(set_)
        0

        >>> bool(set_)
        False

    """

    def __new__(class_, argument=()):
        argument = [_pitch.NumberedPitch(_) for _ in argument]
        argument.sort()
        self = frozenset.__new__(class_, argument)
        return self

    @classmethod
    def _wrap_methods(class_, names):
        def closure(name):
            def inner(self, *arguments):
                result = getattr(super(class_, self), name)(*arguments)
                result = PitchSet(result)
                return result

            inner.fn_name = name
            setattr(class_, name, inner)

        for name in names:
            closure(name)

    def __repr__(self):
        """
        Gets repr.
        """
        if self:
            numbers = [_.number for _ in sorted(self)]
            string = f"{type(self).__name__}({numbers!r})"
        else:
            string = f"{type(self).__name__}()"
        return string

    def __str__(self) -> str:
        """
        Gets string.
        """
        string = ", ".join([str(_.number) for _ in sorted(self)])
        return f"{{{string}}}"


collections.abc.Sequence.register(PitchSet)


PitchSet._wrap_methods(
    [
        "__and__",
        "__or__",
        "__sub__",
        "__xor__",
        "difference",
        "intersection",
        "symmetric_difference",
        "union",
    ]
)


@dataclasses.dataclass(slots=True)
class TwelveToneRow(PitchClassSegment):
    """
    Twelve-tone row.

    ..  container:: example

        Initializes from defaults:

        >>> row = abjad.TwelveToneRow()
        >>> lilypond_file = abjad.illustrate(row)
        >>> abjad.show(lilypond_file) # doctest: +SKIP

        Initializes from integers:

        >>> numbers = [1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8, 0]
        >>> row = abjad.TwelveToneRow(numbers)
        >>> lilypond_file = abjad.illustrate(row)
        >>> abjad.show(lilypond_file) # doctest: +SKIP

        Interpreter representation:

        >>> row
        TwelveToneRow(items=(NumberedPitchClass(1), NumberedPitchClass(11), NumberedPitchClass(9), NumberedPitchClass(3), NumberedPitchClass(6), NumberedPitchClass(7), NumberedPitchClass(5), NumberedPitchClass(4), NumberedPitchClass(10), NumberedPitchClass(2), NumberedPitchClass(8), NumberedPitchClass(0)))

    """

    items: typing.Any = (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11)

    def __post_init__(self):
        assert self.items is not None
        PitchClassSegment.__post_init__(self)
        self._validate_pitch_classes(self)

    def __call__(self, pitch_classes):
        r"""
        Calls row on ``pitch_classes``.

        ..  container:: example

            Example row:

            >>> numbers = [1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8, 0]
            >>> row = abjad.TwelveToneRow(numbers)
            >>> lilypond_file = abjad.illustrate(row)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

        ..  container:: example

            Permutes pitch-classes:

            >>> row([abjad.NumberedPitchClass(2)])
            [NumberedPitchClass(9)]

            >>> row([abjad.NumberedPitchClass(3)])
            [NumberedPitchClass(3)]

            >>> row([abjad.NumberedPitchClass(4)])
            [NumberedPitchClass(6)]

        ..  container:: example

            Permutes pitch-class segment:

            >>> segment = abjad.PitchClassSegment([-2, -1, 6, 7, -1, 7])
            >>> lilypond_file = abjad.illustrate(segment)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> voice = lilypond_file["Voice"]
                >>> string = abjad.lilypond(voice)
                >>> print(string)
                \context Voice = "Voice"
                {
                    bf'8
                    b'8
                    fs'8
                    g'8
                    b'8
                    g'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

            >>> row_ = abjad.TwelveToneRow([10, 0, 2, 6, 8, 7, 5, 3, 1, 9, 4, 11])
            >>> segment = row_(segment)
            >>> segment
            PitchClassSegment([4, 11, 5, 3, 11, 3])

            >>> lilypond_file = abjad.illustrate(segment)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> voice = lilypond_file["Voice"]
                >>> string = abjad.lilypond(voice)
                >>> print(string)
                \context Voice = "Voice"
                {
                    e'8
                    b'8
                    f'8
                    ef'8
                    b'8
                    ef'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Permutes row:

            >>> numbers = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]
            >>> row_2 = abjad.TwelveToneRow(numbers)
            >>> lilypond_file = abjad.illustrate(row_2)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> voice = lilypond_file["Voice"]
                >>> string = abjad.lilypond(voice)
                >>> print(string)
                \context Voice = "Voice"
                {
                    c'8
                    cs'8
                    d'8
                    ef'8
                    e'8
                    f'8
                    fs'8
                    g'8
                    af'8
                    a'8
                    bf'8
                    b'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

            >>> row_3 = row(row_2)
            >>> lilypond_file = abjad.illustrate(row_3)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> voice = lilypond_file["Voice"]
                >>> string = abjad.lilypond(voice)
                >>> print(string)
                \context Voice = "Voice"
                {
                    cs'8
                    b'8
                    a'8
                    ef'8
                    fs'8
                    g'8
                    f'8
                    e'8
                    bf'8
                    d'8
                    af'8
                    c'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Permutes row:

            >>> numbers = [11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0]
            >>> row_2 = abjad.TwelveToneRow(numbers)
            >>> lilypond_file = abjad.illustrate(row_2)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> voice = lilypond_file["Voice"]
                >>> string = abjad.lilypond(voice)
                >>> print(string)
                \context Voice = "Voice"
                {
                    b'8
                    bf'8
                    a'8
                    af'8
                    g'8
                    fs'8
                    f'8
                    e'8
                    ef'8
                    d'8
                    cs'8
                    c'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

            >>> row_3 = row(row_2)
            >>> lilypond_file = abjad.illustrate(row_3)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> voice = lilypond_file["Voice"]
                >>> string = abjad.lilypond(voice)
                >>> print(string)
                \context Voice = "Voice"
                {
                    c'8
                    af'8
                    d'8
                    bf'8
                    e'8
                    f'8
                    g'8
                    fs'8
                    ef'8
                    a'8
                    b'8
                    cs'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Permutes row:

            >>> numbers = [10, 0, 2, 6, 8, 7, 5, 3, 1, 9, 4, 11]
            >>> row_2 = abjad.TwelveToneRow(numbers)
            >>> lilypond_file = abjad.illustrate(row_2)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> voice = lilypond_file["Voice"]
                >>> string = abjad.lilypond(voice)
                >>> print(string)
                \context Voice = "Voice"
                {
                    bf'8
                    c'8
                    d'8
                    fs'8
                    af'8
                    g'8
                    f'8
                    ef'8
                    cs'8
                    a'8
                    e'8
                    b'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

            >>> row_3 = row(row_2)
            >>> lilypond_file = abjad.illustrate(row_3)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> voice = lilypond_file["Voice"]
                >>> string = abjad.lilypond(voice)
                >>> print(string)
                \context Voice = "Voice"
                {
                    af'8
                    cs'8
                    a'8
                    f'8
                    bf'8
                    e'8
                    g'8
                    ef'8
                    b'8
                    d'8
                    fs'8
                    c'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        Returns permuted pitch-classes in object of type ``pitch_classes``.
        """
        new_pitch_classes = []
        for pitch_class in pitch_classes:
            pitch_class = _pitch.NumberedPitchClass(pitch_class)
            i = pitch_class.number
            new_pitch_class = self[i]
            new_pitch_classes.append(new_pitch_class)
        result = type(pitch_classes)(new_pitch_classes)
        return result

    def __getitem__(self, argument):
        r"""
        Gets item or slice identified by ``argument``.

        ..  container:: example

            Example row:

            >>> numbers = [1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8, 0]
            >>> row = abjad.TwelveToneRow(numbers)
            >>> lilypond_file = abjad.illustrate(row)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

        ..  container:: example

            Gets first hexachord:

            >>> lilypond_file = abjad.illustrate(row[:6])
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> voice = lilypond_file["Voice"]
                >>> string = abjad.lilypond(voice)
                >>> print(string)
                \context Voice = "Voice"
                {
                    cs'8
                    b'8
                    a'8
                    ef'8
                    fs'8
                    g'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Gets second hexachord:

            >>> lilypond_file = abjad.illustrate(row[-6:])
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> voice = lilypond_file["Voice"]
                >>> string = abjad.lilypond(voice)
                >>> print(string)
                \context Voice = "Voice"
                {
                    f'8
                    e'8
                    bf'8
                    d'8
                    af'8
                    c'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Gets items in row:

            >>> row = abjad.TwelveToneRow()
            >>> lilypond_file = abjad.illustrate(row)
            >>> abjad.show(lilypond_file) # doctest: +SKIP


            >>> for item in row.items:
            ...     item
            ...
            NumberedPitchClass(0)
            NumberedPitchClass(1)
            NumberedPitchClass(2)
            NumberedPitchClass(3)
            NumberedPitchClass(4)
            NumberedPitchClass(5)
            NumberedPitchClass(6)
            NumberedPitchClass(7)
            NumberedPitchClass(8)
            NumberedPitchClass(9)
            NumberedPitchClass(10)
            NumberedPitchClass(11)

            Gets items in row:

            >>> numbers = [1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8, 0]
            >>> row = abjad.TwelveToneRow(numbers)
            >>> lilypond_file = abjad.illustrate(row)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            >>> for item in row.items:
            ...     item
            ...
            NumberedPitchClass(1)
            NumberedPitchClass(11)
            NumberedPitchClass(9)
            NumberedPitchClass(3)
            NumberedPitchClass(6)
            NumberedPitchClass(7)
            NumberedPitchClass(5)
            NumberedPitchClass(4)
            NumberedPitchClass(10)
            NumberedPitchClass(2)
            NumberedPitchClass(8)
            NumberedPitchClass(0)

        """
        item = self.items.__getitem__(argument)
        try:
            return PitchClassSegment(item)
        except TypeError:
            return item

    def __mul__(self, argument) -> "PitchClassSegment":
        r"""
        Multiplies row by ``argument``.

        ..  container:: example

            Multiplies row:

            >>> row = abjad.TwelveToneRow()
            >>> lilypond_file = abjad.illustrate(row)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            >>> segment = 2 * row
            >>> lilypond_file = abjad.illustrate(segment)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> voice = lilypond_file["Voice"]
                >>> string = abjad.lilypond(voice)
                >>> print(string)
                \context Voice = "Voice"
                {
                    c'8
                    cs'8
                    d'8
                    ef'8
                    e'8
                    f'8
                    fs'8
                    g'8
                    af'8
                    a'8
                    bf'8
                    b'8
                    c'8
                    cs'8
                    d'8
                    ef'8
                    e'8
                    f'8
                    fs'8
                    g'8
                    af'8
                    a'8
                    bf'8
                    b'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Multiplies row:

            >>> numbers = [1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8, 0]
            >>> row = abjad.TwelveToneRow(numbers)
            >>> lilypond_file = abjad.illustrate(segment)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            >>> segment = 2 * row
            >>> lilypond_file = abjad.illustrate(segment)
            >>> abjad.show(lilypond_file) # doctest: +SKIP


            ..  docs::

                >>> voice = lilypond_file["Voice"]
                >>> string = abjad.lilypond(voice)
                >>> print(string)
                \context Voice = "Voice"
                {
                    cs'8
                    b'8
                    a'8
                    ef'8
                    fs'8
                    g'8
                    f'8
                    e'8
                    bf'8
                    d'8
                    af'8
                    c'8
                    cs'8
                    b'8
                    a'8
                    ef'8
                    fs'8
                    g'8
                    f'8
                    e'8
                    bf'8
                    d'8
                    af'8
                    c'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        """
        return PitchClassSegment(self.items) * argument

    def __rmul__(self, argument) -> "PitchClassSegment":
        r"""
        Multiplies ``argument`` by row.

        ..  container:: example

            Multiplies integer by row:

            >>> row = abjad.TwelveToneRow()
            >>> lilypond_file = abjad.illustrate(row)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            >>> segment = row * 2
            >>> lilypond_file = abjad.illustrate(segment)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> voice = lilypond_file["Voice"]
                >>> string = abjad.lilypond(voice)
                >>> print(string)
                \context Voice = "Voice"
                {
                    c'8
                    cs'8
                    d'8
                    ef'8
                    e'8
                    f'8
                    fs'8
                    g'8
                    af'8
                    a'8
                    bf'8
                    b'8
                    c'8
                    cs'8
                    d'8
                    ef'8
                    e'8
                    f'8
                    fs'8
                    g'8
                    af'8
                    a'8
                    bf'8
                    b'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Multiplies integer by row:

            >>> numbers = [1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8, 0]
            >>> row = abjad.TwelveToneRow(numbers)
            >>> lilypond_file = abjad.illustrate(row)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            >>> segment = row * 2
            >>> lilypond_file = abjad.illustrate(segment)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> voice = lilypond_file["Voice"]
                >>> string = abjad.lilypond(voice)
                >>> print(string)
                \context Voice = "Voice"
                {
                    cs'8
                    b'8
                    a'8
                    ef'8
                    fs'8
                    g'8
                    f'8
                    e'8
                    bf'8
                    d'8
                    af'8
                    c'8
                    cs'8
                    b'8
                    a'8
                    ef'8
                    fs'8
                    g'8
                    f'8
                    e'8
                    bf'8
                    d'8
                    af'8
                    c'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        """
        return PitchClassSegment(self.items) * argument

    @property
    def _contents_string(self):
        return ", ".join([str(abs(pc)) for pc in self])

    @staticmethod
    def _validate_pitch_classes(pitch_classes):
        numbers = [pc.number for pc in pitch_classes]
        numbers.sort()
        if not numbers == list(range(12)):
            message = f"must contain all twelve pitch-classes: {pitch_classes!r}."
            raise ValueError(message)

    def invert(self, axis=None) -> "TwelveToneRow":
        r"""
        Inverts row about optional ``axis``.

        ..  container:: example

            Example row:

            >>> numbers = [1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8, 0]
            >>> row = abjad.TwelveToneRow(numbers)
            >>> lilypond_file = abjad.illustrate(row)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

        ..  container:: example

            Inverts row about first pitch-class when ``axis`` is none:

            >>> inversion = row.invert()
            >>> lilypond_file = abjad.illustrate(inversion)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> voice = lilypond_file["Voice"]
                >>> string = abjad.lilypond(voice)
                >>> print(string)
                \context Voice = "Voice"
                {
                    cs'8
                    ef'8
                    f'8
                    b'8
                    af'8
                    g'8
                    a'8
                    bf'8
                    e'8
                    c'8
                    fs'8
                    d'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

            First pitch-classes are equal:

            >>> row[0] == inversion[0]
            True

        ..  container:: example

            Inverts row about pitch-class 1:

            >>> inversion = row.invert(axis=1)
            >>> lilypond_file = abjad.illustrate(inversion)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> voice = lilypond_file["Voice"]
                >>> string = abjad.lilypond(voice)
                >>> print(string)
                \context Voice = "Voice"
                {
                    cs'8
                    ef'8
                    f'8
                    b'8
                    af'8
                    g'8
                    a'8
                    bf'8
                    e'8
                    c'8
                    fs'8
                    d'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

            Same result as above.

        ..  container:: example

            Inverts row about pitch-class 0:

            >>> inversion = row.invert(axis=0)
            >>> lilypond_file = abjad.illustrate(inversion)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> voice = lilypond_file["Voice"]
                >>> string = abjad.lilypond(voice)
                >>> print(string)
                \context Voice = "Voice"
                {
                    b'8
                    cs'8
                    ef'8
                    a'8
                    fs'8
                    f'8
                    g'8
                    af'8
                    d'8
                    bf'8
                    e'8
                    c'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Inverts row about pitch-class 5:

            >>> inversion = row.invert(axis=5)
            >>> lilypond_file = abjad.illustrate(inversion)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> voice = lilypond_file["Voice"]
                >>> string = abjad.lilypond(voice)
                >>> print(string)
                \context Voice = "Voice"
                {
                    a'8
                    b'8
                    cs'8
                    g'8
                    e'8
                    ef'8
                    f'8
                    fs'8
                    c'8
                    af'8
                    d'8
                    bf'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        """
        if axis is None:
            axis = self[0]
        items = [pc.invert(axis=axis) for pc in self]
        return dataclasses.replace(self, items=items)

    def multiply(self, n=1) -> "TwelveToneRow":
        r"""
        Multiplies pitch-classes in row by ``n``.

        ..  container:: example

            Example row:

            >>> numbers = [1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8, 0]
            >>> row = abjad.TwelveToneRow(numbers)
            >>> lilypond_file = abjad.illustrate(row)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

        ..  container:: example

            Multiplies pitch-classes in row by 5:

            >>> multiplication = row.multiply(n=5)
            >>> lilypond_file = abjad.illustrate(multiplication)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> voice = lilypond_file["Voice"]
                >>> string = abjad.lilypond(voice)
                >>> print(string)
                \context Voice = "Voice"
                {
                    f'8
                    g'8
                    a'8
                    ef'8
                    fs'8
                    b'8
                    cs'8
                    af'8
                    d'8
                    bf'8
                    e'8
                    c'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Multiplies pitch-classes in row by 7:

            >>> multiplication = row.multiply(n=7)
            >>> lilypond_file = abjad.illustrate(multiplication)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> voice = lilypond_file["Voice"]
                >>> string = abjad.lilypond(voice)
                >>> print(string)
                \context Voice = "Voice"
                {
                    g'8
                    f'8
                    ef'8
                    a'8
                    fs'8
                    cs'8
                    b'8
                    e'8
                    bf'8
                    d'8
                    af'8
                    c'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Multiplies pitch-classes in row by 1:

            >>> multiplication = row.multiply(n=1)
            >>> lilypond_file = abjad.illustrate(multiplication)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> voice = lilypond_file["Voice"]
                >>> string = abjad.lilypond(voice)
                >>> print(string)
                \context Voice = "Voice"
                {
                    cs'8
                    b'8
                    a'8
                    ef'8
                    fs'8
                    g'8
                    f'8
                    e'8
                    bf'8
                    d'8
                    af'8
                    c'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        """
        return type(self)(PitchClassSegment.multiply(self, n=n))

    def retrograde(self) -> "TwelveToneRow":
        r"""
        Gets retrograde of row.

        ..  container:: example

            Example row:

            >>> numbers = [1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8, 0]
            >>> row = abjad.TwelveToneRow(numbers)
            >>> lilypond_file = abjad.illustrate(row)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

        ..  container:: example

            Gets retrograde of row:

            >>> retrograde = row.retrograde()
            >>> lilypond_file = abjad.illustrate(retrograde)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> voice = lilypond_file["Voice"]
                >>> string = abjad.lilypond(voice)
                >>> print(string)
                \context Voice = "Voice"
                {
                    c'8
                    af'8
                    d'8
                    bf'8
                    e'8
                    f'8
                    g'8
                    fs'8
                    ef'8
                    a'8
                    b'8
                    cs'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Gets retrograde of retrograde of row:

            >>> retrograde = row.retrograde().retrograde()
            >>> lilypond_file = abjad.illustrate(retrograde)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> voice = lilypond_file["Voice"]
                >>> string = abjad.lilypond(voice)
                >>> print(string)
                \context Voice = "Voice"
                {
                    cs'8
                    b'8
                    a'8
                    ef'8
                    fs'8
                    g'8
                    f'8
                    e'8
                    bf'8
                    d'8
                    af'8
                    c'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

            >>> retrograde == row
            True

        """
        return type(self)(PitchClassSegment.retrograde(self))

    def rotate(self, n=0) -> "TwelveToneRow":
        r"""
        Rotates row by index ``n``.

        ..  container:: example

            Example row:

            >>> numbers = [1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8, 0]
            >>> row = abjad.TwelveToneRow(numbers)
            >>> lilypond_file = abjad.illustrate(row)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

        ..  container:: example

            Rotates row to the right:

            >>> rotation = row.rotate(n=1)
            >>> lilypond_file = abjad.illustrate(rotation)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> voice = lilypond_file["Voice"]
                >>> string = abjad.lilypond(voice)
                >>> print(string)
                \context Voice = "Voice"
                {
                    c'8
                    cs'8
                    b'8
                    a'8
                    ef'8
                    fs'8
                    g'8
                    f'8
                    e'8
                    bf'8
                    d'8
                    af'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Rotates row to the left:

            >>> rotation = row.rotate(n=-1)
            >>> lilypond_file = abjad.illustrate(rotation)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> voice = lilypond_file["Voice"]
                >>> string = abjad.lilypond(voice)
                >>> print(string)
                \context Voice = "Voice"
                {
                    b'8
                    a'8
                    ef'8
                    fs'8
                    g'8
                    f'8
                    e'8
                    bf'8
                    d'8
                    af'8
                    c'8
                    cs'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Rotates row by zero:

            >>> rotation = row.rotate(n=0)
            >>> lilypond_file = abjad.illustrate(rotation)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> voice = lilypond_file["Voice"]
                >>> string = abjad.lilypond(voice)
                >>> print(string)
                \context Voice = "Voice"
                {
                    cs'8
                    b'8
                    a'8
                    ef'8
                    fs'8
                    g'8
                    f'8
                    e'8
                    bf'8
                    d'8
                    af'8
                    c'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

            >>> rotation == row
            True

        """
        return type(self)(PitchClassSegment.rotate(self, n=n))

    def transpose(self, n=0) -> "TwelveToneRow":
        r"""
        Transposes row by index ``n``.

        ..  container:: example

            Example row:

            >>> numbers = [1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8, 0]
            >>> row = abjad.TwelveToneRow(numbers)
            >>> lilypond_file = abjad.illustrate(row)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

        ..  container:: example

            Transposes row by positive index:

            >>> transposition = row.transpose(n=13)
            >>> lilypond_file = abjad.illustrate(transposition)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> voice = lilypond_file["Voice"]
                >>> string = abjad.lilypond(voice)
                >>> print(string)
                \context Voice = "Voice"
                {
                    d'8
                    c'8
                    bf'8
                    e'8
                    g'8
                    af'8
                    fs'8
                    f'8
                    b'8
                    ef'8
                    a'8
                    cs'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Transposes row by negative index:

            >>> transposition = row.transpose(n=-13)
            >>> lilypond_file = abjad.illustrate(transposition)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> voice = lilypond_file["Voice"]
                >>> string = abjad.lilypond(voice)
                >>> print(string)
                \context Voice = "Voice"
                {
                    c'8
                    bf'8
                    af'8
                    d'8
                    f'8
                    fs'8
                    e'8
                    ef'8
                    a'8
                    cs'8
                    g'8
                    b'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Transposes row by zero index:

            >>> transposition = row.transpose(n=0)
            >>> lilypond_file = abjad.illustrate(transposition)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> voice = lilypond_file["Voice"]
                >>> string = abjad.lilypond(voice)
                >>> print(string)
                \context Voice = "Voice"
                {
                    cs'8
                    b'8
                    a'8
                    ef'8
                    fs'8
                    g'8
                    f'8
                    e'8
                    bf'8
                    d'8
                    af'8
                    c'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

            >>> transposition == row
            True

        """
        return type(self)(PitchClassSegment.transpose(self, n=n))


def _get_most_compact_ordering(candidates):
    widths = []
    for candidate in candidates:
        if candidate[0] < candidate[-1]:
            width = abs(candidate[-1] - candidate[0])
        else:
            width = abs(candidate[-1] + 12 - candidate[0])
        widths.append(width)
    minimum_width = min(widths)
    candidates_ = []
    for candidate, width in zip(candidates, widths):
        if width == minimum_width:
            candidates_.append(candidate)
    candidates = candidates_
    assert 1 <= len(candidates)
    if len(candidates) == 1:
        segment = candidates[0]
        segment = PitchClassSegment(segment)
        return segment
    for i in range(len(candidates[0]) - 1):
        widths = []
        for candidate in candidates:
            stop = i + 1
            if candidate[0] < candidate[stop]:
                width = abs(candidate[stop] - candidate[0])
            else:
                width = abs(candidate[stop] + 12 - candidate[0])
            widths.append(width)
        minimum_width = min(widths)
        candidates_ = []
        for candidate, width in zip(candidates, widths):
            if width == minimum_width:
                candidates_.append(candidate)
        candidates = candidates_
        if len(candidates) == 1:
            segment = candidates[0]
            segment = PitchClassSegment(segment)
            return segment
    candidates.sort(key=lambda x: x[0])
    segment = candidates[0]
    segment = PitchClassSegment(segment)
    return segment


def make_interval_class_vector(pitches):
    """
    Makes interval-class vector.
    """
    intervals = []
    pairs = _enumerate.yield_pairs(list(pitches))
    for first, second in pairs:
        intervals.append(second - first)
    items = [_pitch.NumberedInversionEquivalentIntervalClass(_) for _ in intervals]
    counter = collections.Counter(items)
    counts = []
    for i in range(7):
        item = _pitch.NumberedInversionEquivalentIntervalClass(i)
        count = counter.get(item, 0)
        counts.append(count)
    counts = "".join([str(_) for _ in counts])
    if len(counter) == 13:
        quartertones = []
        for i in range(6):
            item = _pitch.NumberedInversionEquivalentIntervalClass(i + 0.5)
            count = counter.get(item, 0)
            quartertones.append(count)
        quartertones = "".join([str(_) for _ in quartertones])
        string = rf'\tiny \column {{ "{counts}" "{quartertones}" }}'
    else:
        string = rf"\tiny {counts}"
    return string


def voice_horizontally(
    pcs: typing.Sequence[_pitch.NamedPitchClass],
    initial_octave: _pitch.Octave = _pitch.Octave(4),
) -> tuple[_pitch.NamedPitch, ...]:
    r"""
    Voices named ``pcs`` with each pitch as close to the previous pitch as possible.

    ..  container:: example

        >>> pcs = [abjad.NamedPitchClass(_) for _ in "c b d e f g e b a c".split()]
        >>> pitches = abjad.pcollections.voice_horizontally(pcs)
        >>> staff = abjad.Staff([abjad.Note(_, (1, 8)) for _ in pitches])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> string = abjad.lilypond(staff)
            >>> print(string)
            \new Staff
            {
                c'8
                b8
                d'8
                e'8
                f'8
                g'8
                e'8
                b8
                a8
                c'8
            }

    """
    assert all(isinstance(_, _pitch.NamedPitchClass) for _ in pcs), repr(pcs)
    assert isinstance(initial_octave, _pitch.Octave), repr(initial_octave)
    pitches = []
    if pcs:
        pc = pcs[0]
        pitch = _pitch.NamedPitch((pc.name, initial_octave))
        pitches.append(pitch)
        for pc in pcs[1:]:
            number = _pitch.NamedPitch((pc.name, initial_octave)).number
            semitones = abs((number - pitches[-1].number))
            while 6 < semitones:
                if number < pitches[-1].number:
                    number += 12
                else:
                    number -= 12
                semitones = abs((number - pitches[-1].number))
            pitch = _pitch.NamedPitch(number)
            pitches.append(pitch)
    return tuple(pitches)


def voice_vertically(
    pcs: typing.Sequence[_pitch.NamedPitchClass],
    initial_octave: _pitch.Octave = _pitch.Octave(4),
) -> tuple[_pitch.NamedPitch, ...]:
    r"""
    Voices named ``pcs`` with each pitch not lower than the previous.

    ..  container:: example

        >>> pcs = [abjad.NamedPitchClass(_) for _ in "c ef g bf d f af".split()]
        >>> pitches = abjad.pcollections.voice_vertically(pcs)
        >>> staff = abjad.Staff([abjad.Note(_, (1, 8)) for _ in pitches])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> string = abjad.lilypond(staff)
            >>> print(string)
            \new Staff
            {
                c'8
                ef'8
                g'8
                bf'8
                d''8
                f''8
                af''8
            }

    """
    assert all(isinstance(_, _pitch.NamedPitchClass) for _ in pcs), repr(pcs)
    assert isinstance(initial_octave, _pitch.Octave), repr(initial_octave)
    pitches = []
    if pcs:
        pc = _pitch.NamedPitchClass(pcs[0])
        pitch = _pitch.NamedPitch((pc.name, initial_octave))
        pitches.append(pitch)
        for pc in pcs[1:]:
            pc = _pitch.NamedPitchClass(pc)
            pitch = _pitch.NamedPitch((pc.name, initial_octave))
            while pitch < pitches[-1]:
                pitch += 12
            pitches.append(pitch)
    return tuple(pitches)



import abjad


def test_LogicalTie_written_duration_01():
    staff = abjad.Staff("c' ~ c'16")

    assert abjad.get.logical_tie(staff[0]).written_duration == abjad.Duration(5, 16)


def test_LogicalTie_written_duration_02():
    staff = abjad.Staff("c'")

    assert abjad.get.logical_tie(staff[0]).written_duration == abjad.Duration(1, 4)



import copy
import inspect
import pickle

import pytest

import abjad

classes = abjad.list_all_classes()


class_to_default_values = {
    abjad.io.AbjadGrapher: (abjad.Note("c'4"),),
    abjad.io.Illustrator: (abjad.Note("c'4"),),
    abjad.io.LilyPondIO: (abjad.Note("c'4"),),
    abjad.io.Player: (abjad.Note("c'4"),),
    abjad.parsers.parser.MarkupCommand: (r"\hcenter-in",),
    abjad.Articulation: ("staccato",),
    abjad.Bundle: (abjad.Articulation("."),),
    abjad.ColorFingering: (0,),
    abjad.InstrumentName: ("Piano",),
    abjad.Markup: (r"\markup Allegro",),
    abjad.MetricModulation: (abjad.Note("c'4"), abjad.Note("c'4.")),
    abjad.MetronomeMark: (abjad.Duration(1, 4), 90),
    abjad.TextMark: (r'\textMark \markup \italic "V.S."',),
    abjad.ShortInstrumentName: (r"\markup Vc.",),
    abjad.StaffChange: ("RH_Staff",),
    abjad.TimeSignature: ((4, 4),),
    abjad.Tweak: (r"\tweak color #red",),
}


@pytest.mark.parametrize("class_", classes)
def test_abjad___copy___(class_):
    """
    All concrete classes can copy.
    """
    if inspect.isabstract(class_):
        return
    if getattr(class_, "_is_abstract", None) is True:
        return
    if issubclass(class_, Exception):
        return
    default_values = class_to_default_values.get(class_, ())
    instance_1 = class_(*default_values)
    instance_2 = copy.copy(instance_1)
    assert instance_2 is not None
    assert instance_1 == instance_2


@pytest.mark.parametrize("class_", classes)
def test_abjad___deepcopy___(class_):
    """
    All concrete classes can deepcopy.
    """
    if inspect.isabstract(class_):
        return
    if getattr(class_, "_is_abstract", None) is True:
        return
    if issubclass(class_, Exception):
        return
    default_values = class_to_default_values.get(class_, ())
    instance_1 = class_(*default_values)
    instance_2 = copy.deepcopy(instance_1)
    assert instance_2 is not None
    assert instance_1 == instance_2


@pytest.mark.parametrize("class_", classes)
def test_abjad___hash___(class_):
    """
    All concrete classes with __hash__ can hash.
    """
    if inspect.isabstract(class_):
        return
    if getattr(class_, "_is_abstract", None) is True:
        return
    if getattr(class_, "__hash__", None) is None:
        return
    default_values = class_to_default_values.get(class_, ())
    instance = class_(*default_values)
    value = hash(instance)
    assert isinstance(value, int)


@pytest.mark.parametrize("class_", classes)
def test_abjad___repr___(class_):
    """
    All concrete classes have a repr and a string.
    """
    _allowed_to_be_empty_string = (
        abjad.Accidental,
        abjad.Tag,
    )
    if inspect.isabstract(class_):
        return
    if getattr(class_, "_is_abstract", None) is True:
        return
    default_values = class_to_default_values.get(class_, ())
    instance = class_(*default_values)
    string = repr(instance)
    assert string is not None
    assert string != ""
    if issubclass(class_, Exception):
        return
    string = str(instance)
    assert string is not None
    if class_ not in _allowed_to_be_empty_string:
        assert string != ""


@pytest.mark.parametrize("class_", classes)
def test_abjad___rmul___(class_):
    """
    All classes with __mul__ also implement __rmul__.
    """
    if inspect.isabstract(class_):
        return
    if hasattr(class_, "__mul__"):
        assert hasattr(class_, "__rmul__")


@pytest.mark.parametrize("class_", classes)
def test_abjad___slots___(class_):
    """
    All classes with __slots__ implement __slots__ correctly.
    """
    if inspect.isabstract(class_):
        return
    if getattr(class_, "_is_abstract", None) is True:
        return
    attrs = inspect.classify_class_attrs(class_)
    attrs = dict((attr.name, attr) for attr in attrs)
    if "__slots__" not in attrs:
        return
    elif attrs["__slots__"].defining_class is not class_:
        return
    default_values = class_to_default_values.get(class_, ())
    instance_1 = class_(*default_values)
    assert not hasattr(instance_1, "__dict__")


@pytest.mark.parametrize("class_", classes)
def test_abjad_contexted_persistence(class_):
    """
    All classes with "persistent" property also have "context" property.
    """
    if inspect.isabstract(class_):
        return
    if hasattr(class_, "persistent"):
        assert hasattr(class_, "context")


@pytest.mark.parametrize("class_", classes)
def test_abjad_format_site_defaults(class_):
    """
    All classes with "site" property default to "before" or "after".
    """
    if inspect.isabstract(class_):
        return
    if hasattr(class_, "site"):
        default_values = class_to_default_values.get(class_, ())
        instance = class_(*default_values)
        assert instance.site in ("before", "after"), repr(class_.site)


@pytest.mark.parametrize("class_", classes)
def test_abjad_pickle(class_):
    """
    All concrete classes are pickable.
    """
    if inspect.isabstract(class_):
        return
    if getattr(class_, "_is_abstract", None) is True:
        return
    if issubclass(class_, Exception):
        return
    default_values = class_to_default_values.get(class_, ())
    instance_1 = class_(*default_values)
    pickle_string = pickle.dumps(instance_1)
    instance_2 = pickle.loads(pickle_string)
    assert instance_1 == instance_2
